# *MACHINING_DATA.TCL 
###############################################################################
# machining_DATA.tcl - DBC Event Handler for database stored as ascii file
###############################################################################
# Revisions
# Date          Who                     Reason
# 22Oct2003     rlm                     Initial
# 26Jan2004     rlm                     Return integers for cut feed percentages
# 26Jan2004     rlm                     remove rest of debugging code
# 30Jan2004     rlm                     Move ASC_write to dbc_ascii_general
# 12Feb2004     rlm                     Add library revision information
################################################################################
#
# The format of each line in the text file is:
#
#         libref  optype ptmtl tlmtl speed feed
#
#         libref   is a string
#         optype   is "Milling" "Turning" "Drilling"
#         ptmtl    is  code number for now 
#         tlmtl    is "HSS" "Carbide" "CBN"
#         depth    is a double
#         speed    is a double
#         feed     is a double
#
# The DB_ID of a DB_ALIAS is its field number in the line (0-based)
###############################################################################
#MOM_set_debug_mode ON
#---------------------------------------------
# Non DBC Global Variables
#---------------------------------------------
#set dbFileName     "[MOM_ask_env_var UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR]machining_data.dat" ; # The ASCII Database File
set DEBUG 1
set fPointer ""

#---------------------------------------------
# Global variables set by DBC for Input/Output
#---------------------------------------------
set dbc_lhs_exp     ""
set dbc_rhs_exp     ""
set dbc_relop       ""
set dbc_query       ""
set dbc_subqry1     ""
set dbc_subqry2     ""
set dbc_boolop      ""
set dbc_class_name  ""
set dbc_attr_count  0
set dbc_attr_id     ""
set dbc_query_count 0
set dbc_libref      ""
set dbc_var_list    ""
set dbc_tool_diameter ""
set dbc_tool_height ""
set dbc_part_units  "english"  ; #   ( english  : 0  , metric : 1)
set dbc_cutmthd_libref  ""
set dbc_part_material_libref ""
set dbc_tool_material_libref ""

#
# Express feedrates for the following feed types
# as percentages of Cut feedrate. 
# A value of 0 indicates no relation to Cut feedrate.
#
set dbc_engage_percent     90
set dbc_first_cut_percent  60
set dbc_approach_percent    0 
set dbc_stepover_percent    0 
set dbc_retract_percent     0 
set dbc_departure_percent   0 
set dbc_return_percent      0 
set asc_debug    0

proc ASC_t_create_filename {env_var_name filename} \
{
#
# Creates a complete filename by evaluating an environment variable
# for the directory information
#
  set env_var [MOM_ask_env_var $env_var_name]

  if { $env_var == "" } \
    {
         set message "Can't read environment variable $env_var_name"
         MOM_abort "\n $message"
    }

    set fname ""
    set fname [append fname $env_var $filename]

    return $fname

}

#---------------------------------------------
proc MOM__boot {} {
#---------------------------------------------

# source some general procedures

#
# dbc_ascii_general.tcl
#
  set filename \
    [ASC_t_create_filename "UGII_UG_LIBRARY_DIR" "dbc_ascii_general.tcl"]


  if {  [catch {source $filename}] == "1" } \
   {
      set message "Can't load .tcl file: $filename"
      MOM_abort "\n $message"
   }

}

proc MOM__halt {} \
{

}
#---------------------------------------------
proc DBC_init_db {} {
#---------------------------------------------
   global DEBUG
   global dataBase
   global dbFileName
   global fPointer
   global asc_file_name
   global asc_database_count

   set asc_file_name ""
   set asc_part_units ""
   set data_unit 0
   set mode 0

#  
#  Set the filename for ASCII Data File
#
   ASC_set_data_file_name   

#  Changed to improve performance
#  And load the file into memory
#
    ASC_load_data_file $asc_file_name $data_unit $mode

}

#---------------------------------------------
proc ASC_set_data_file_name {} {
#---------------------------------------------
   global dbc_part_units
   global asc_file_name
   global asc_part_units

   MOM_ask_part_units  ;# writes to dbc_part_units

   set env_var UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR
   set asc_file_name [ASC_t_create_filename "$env_var" "machining_data.dat"]
   set asc_part_units $dbc_part_units
}

#---------------------------------------------
proc ASC_get_tool_size { DB_ROW DIAMETER LENGTH } {
#---------------------------------------------
    upvar $DB_ROW db_row
    upvar $DIAMETER asc_tool_diameter
    upvar $LENGTH asc_tool_length

    global dbc_part_units

    if { $dbc_part_units == "english"} \
    {
        set asc_tool_diameter [ASC_ask_att_val DIAM_IN $db_row \
                 "%g" 0 flag]
        set asc_tool_length [ASC_ask_att_val LENGTH_IN $db_row \
                 "%g" 0 flag]
    } else \
    {
        set asc_tool_diameter [ASC_ask_att_val DIAM_MM $db_row \
                 "%g" 0 flag]
        set asc_tool_length [ASC_ask_att_val LENGTH_MM $db_row \
                 "%g" 0 flag]
    }
}

#---------------------------------------------
proc ASC_get_machining_data { DB_ROW CUT_DEPTH STEPOVER FEED_VALUE SPEED_VALUE } {
#---------------------------------------------
    upvar $DB_ROW db_row
    upvar $CUT_DEPTH asc_cut_value
    upvar $STEPOVER asc_stepover
    upvar $FEED_VALUE asc_feed_value
    upvar $SPEED_VALUE asc_speed_value

    global dbc_part_units

    if { $dbc_part_units == "english"} \
    {
        set asc_cut_value [ASC_ask_att_val DPT_CUT_IN $db_row \
                 "%g" 0 flag]
        set asc_stepover [ASC_ask_att_val STEPOVER_IN $db_row \
                 "%g" 0 flag]
        set asc_speed_value [ASC_ask_att_val SURF_SPEED_FPM $db_row \
                 "%g" 0 flag]
        set asc_feed_value [ASC_ask_att_val FEED_IPT $db_row \
                 "%g" 0 flag]
    } else \
    {
        set asc_cut_value [ASC_ask_att_val DPT_CUT_MM $db_row \
                 "%g" 0 flag]
        set asc_stepover [ASC_ask_att_val STEPOVER_MM $db_row \
                 "%g" 0 flag]
        set asc_speed_value [ASC_ask_att_val SURF_SPEED_MPM $db_row \
                 "%g" 0 flag]
        set asc_feed_value [ASC_ask_att_val FEED_MMPT $db_row \
                 "%g" 0 flag]
    }
}

#---------------------------------------------
proc ASC_get_feedrate_percentages { DB_ROW APPROACH_PERCENT ENGAGE_PERCENT \
    FIRSTCUT_PERCENT STEPOVER_PERCENT RETRACT_PERCENT RETURN_PERCENT DEPART_PERCENT } {
#---------------------------------------------
    upvar $DB_ROW db_row
    upvar $APPROACH_PERCENT asc_approach_pct
    upvar $ENGAGE_PERCENT asc_engage_pct
    upvar $FIRSTCUT_PERCENT asc_firstcut_pct
    upvar $STEPOVER_PERCENT asc_stepover_pct
    upvar $RETRACT_PERCENT asc_retract_pct
    upvar $RETURN_PERCENT asc_return_pct
    upvar $DEPART_PERCENT asc_depart_pct

    set asc_approach_pct [ASC_ask_att_val APPROACH_PCT $db_row \
                   "%d" 0 flag]
    set asc_engage_pct [ASC_ask_att_val ENGAGE_PCT $db_row \
                   "%d" 0 flag]
    set asc_firstcut_pct [ASC_ask_att_val FRSTCUT_PCT $db_row \
                   "%d" 0 flag]
    set asc_stepover_pct [ASC_ask_att_val STEPOVER_PCT $db_row \
                   "%d" 0 flag]
    set asc_retract_pct [ASC_ask_att_val RETRACT_PCT $db_row \
                   "%d" 0 flag]
    set asc_return_pct [ASC_ask_att_val RETURN_PCT $db_row \
                   "%d" 0 flag]
    set asc_depart_pct [ASC_ask_att_val DEPART_PCT $db_row \
                   "%d" 0 flag]
}

#------------------------------------------------------------------------
proc DBC_create_query { } {
#------------------------------------------------------------------------
  global DEBUG
  global dbc_query
  global dbc_subqry1
  global dbc_subqry2
  global dbc_boolop

# set dbc_query "$dbc_subqry1 $dbc_boolop $dbc_subqry2"

  ASC_create_query
}

#------------------------------------------------------------------------
proc DBC_execute_query { } {
#------------------------------------------------------------------------
global asc_database_count

ASC_execute_query

}

#---------------------------------------------
proc DBC_retrieve {} {
#---------------------------------------------
    global DEBUG
    global dbc_event_error  
    global dbc_part_units  
    global dbc_cutmthd_libref
    global dbc_part_material_libref
    global dbc_tool_material_libref
    global dbc_tool_diameter
    global dbc_tool_length

    global dbc_surface_speed 
    global dbc_feed_per_tooth
    global dbc_depth_of_cut
    global dbc_stepover

    global dbc_approach_percent
    global dbc_engage_percent
    global dbc_first_cut_percent
    global dbc_stepover_percent
    global dbc_retract_percent
    global dbc_return_percent
    global dbc_departure_percent
    global fPointer
    global dbFileName
    global asc_database_count
    global asc_database
    global asc_file_name
    global file_unit
    global asc_units
    global asc_next_line
    global dbc_libref

    set index 0
    set low_record_indx -1
    set low_ratio_delta 999
    set high_record_indx -1
    set high_ratio_delta 999

    set initialize_flag 1
    set asc_data_file_name $asc_file_name
    set asc_next_line ""

    if { $dbc_part_units == "english" } \
    {
        set file_unit $asc_units(inch)
    } else \
    {
        set file_unit $asc_units(mm)
    }

## Changed function to  improve performance
    
#    set ret_code [catch {open $asc_data_file_name "r"} fp]
#    if { $ret_code } \
#    {
#        return 1
#    }

#    ASC_init_database

#  compute the D4/L3 ratio for the input tool
    if { $dbc_tool_length > 0 } \
    {
        set input_ratio [expr pow($dbc_tool_length,3) / pow($dbc_tool_diameter,4)]
#        set input_ratio [expr pow($dbc_tool_diameter,4) / pow($dbc_tool_length,3)]
    } else \
    {
        set input_ratio 0
    }
#
# Loop over all the DATA records
#
# The db_row variable has to be initialized (see also ASC_translate_att_alias)

    set db_row 0
    while { $db_row < $asc_database_count } \
    {
        set ascii_cutmethd_libref [ ASC_ask_att_val OPERTYPE  \
                                    $db_row "" 0 flag] 

        set ascii_part_material_libref [ASC_ask_att_val PARTMAT $db_row \
                                                   "" 0 flag]

        set ascii_tool_material_libref [ASC_ask_att_val TOOLMAT $db_row \
                                                   "" 0 flag]

# See if row matches primary selection criteria
        if { $dbc_cutmthd_libref == $ascii_cutmethd_libref && \
            $dbc_part_material_libref == $ascii_part_material_libref && \
              $dbc_tool_material_libref == $ascii_tool_material_libref }  \
        {
#  It does.  Get tool size and check for match
            ASC_get_tool_size db_row local_tool_diameter local_tool_length
            if { $dbc_tool_diameter == $local_tool_diameter && \
                 $dbc_tool_length == $local_tool_length } \
            {
#  Got a matching record, get remaining data and return
                ASC_get_machining_data db_row dbc_depth_of_cut dbc_stepover \
                   dbc_feed_per_tooth dbc_surface_speed 

                ASC_get_feedrate_percentages db_row dbc_approach_percent \
                    dbc_engage_percent dbc_first_cut_percent dbc_stepover_percent \
                    dbc_retract_percent dbc_return_percent dbc_departure_percent

                return 0

            } else \
            {
##  Exact match not found check D4/L3 ratio for possible interpolation consideration
                if {$local_tool_length > 0} \
                {
                     set local_ratio [expr pow($local_tool_length,3) / pow($local_tool_diameter,4)]
#                    set local_ratio [expr pow($local_tool_diameter,4) / pow($local_tool_length,3)]
                } else \
                {
                    set local_ratio 9999
                }
                if { $local_ratio < $input_ratio } \
                {
                    set local_delta [expr $input_ratio - $local_ratio]
                    if { $local_delta < $low_ratio_delta } \
                    {
#  This record is closer than previous low record.  Save its data
                        set low_record_indx $db_row
                        set low_ratio_delta $local_delta
                    }
                } elseif { $local_ratio > $input_ratio } \
                {
                    set local_delta [expr $local_ratio - $input_ratio]
                    if { $local_delta < $high_ratio_delta } \
                    {
#  This record is closer than previous high record.  Save its data
                        set high_record_indx $db_row
                        set high_ratio_delta $local_delta
                    }
                }
            }
        }
        incr db_row
    }

#  Check if we have found any matches to primary criteria
    if { $low_record_indx == -1 && $high_record_indx == -1} \
    {
#  not matches found at all -- return error message
         set dbc_event_error "No appropriate machining data found"
    } elseif { $low_record_indx == -1 } \
    {
#  have a match on the high end, use it
        ASC_get_machining_data high_record_indx dbc_depth_of_cut dbc_stepover \
           dbc_feed_per_tooth dbc_surface_speed 

        ASC_get_feedrate_percentages high_record_indx dbc_approach_percent \
            dbc_engage_percent dbc_first_cut_percent dbc_stepover_percent \
            dbc_retract_percent dbc_return_percent dbc_departure_percent
    } elseif { $high_record_indx == -1 } \
    {
#  have a match on the low end, use it
        ASC_get_machining_data low_record_indx dbc_depth_of_cut dbc_stepover \
           dbc_feed_per_tooth dbc_surface_speed 

        ASC_get_feedrate_percentages low_record_indx dbc_approach_percent \
            dbc_engage_percent dbc_first_cut_percent dbc_stepover_percent \
            dbc_retract_percent dbc_return_percent dbc_departure_percent
    } else \
    {
#  have matches on both sides, interpolate between data values
        ASC_get_machining_data low_record_indx low_depth_of_cut low_stepover \
           low_feed_per_tooth low_surface_speed 

        ASC_get_feedrate_percentages low_record_indx low_approach_percent \
            low_engage_percent low_first_cut_percent low_stepover_percent \
            low_retract_percent low_return_percent low_departure_percent

        ASC_get_machining_data high_record_indx high_depth_of_cut high_stepover \
           high_feed_per_tooth high_surface_speed 

        ASC_get_feedrate_percentages high_record_indx high_approach_percent \
            high_engage_percent high_first_cut_percent high_stepover_percent \
            high_retract_percent high_return_percent high_departure_percent

        set step [expr $low_ratio_delta / ($high_ratio_delta + $low_ratio_delta)]

        set dbc_depth_of_cut [expr $low_depth_of_cut + $step * ($high_depth_of_cut - $low_depth_of_cut)]
        set dbc_stepover [expr $low_stepover+ $step * ($high_stepover - $low_stepover)]
        set dbc_feed_per_tooth [expr $low_feed_per_tooth + $step * ($high_feed_per_tooth - $low_feed_per_tooth)]
        set dbc_surface_speed [expr $low_surface_speed + $step * ($high_surface_speed - $low_surface_speed)]
        set dbc_approach_percent [expr int ([expr $low_approach_percent + \
            $step * ($high_approach_percent - $low_approach_percent)])]
        set dbc_engage_percent [expr int ([expr $low_engage_percent + \
            $step * ($high_engage_percent - $low_engage_percent)])]
        set dbc_first_cut_percent [expr int ([expr $low_first_cut_percent + \
            $step * ($high_first_cut_percent - $low_first_cut_percent)])]
        set dbc_stepover_percent [expr int ([expr $low_stepover_percent + \
            $step * ($high_stepover_percent - $low_stepover_percent)])]
        set dbc_retract_percent [expr int ([expr $low_retract_percent + \
            $step * ($high_retract_percent - $low_retract_percent)])]
        set dbc_return_percent [expr int ([expr $low_return_percent + \
            $step * ($high_return_percent - $low_return_percent)])]
        set dbc_departure_percent [expr int ([expr $low_departure_percent + \
            $step * ($high_departure_percent - $low_departure_percent)])]
    }   
}

#------------------------------------------------------------------------
proc DBC_write { } {
#------------------------------------------------------------------------
global asc_database_name

    set asc_database_name "machining_data.dat"
    ASC_write
}

#------------------------------------------------------------------------
proc ASC_output_header { fileid } {
#------------------------------------------------------------------------
global dbc_attr_count
global dbc_attr_aliases
global dbc_attr_id
global dbc_logname


    set daytime [clock seconds]
    set out_daytime [clock format $daytime -format "%a %b %d %Y %I:%M %p"]

    set headerMessage "# MACHINING_DATA.DAT
##########################################################################
#
#   PURPOSE:
#
#      This is the database file used for defining Machining Data in
#      an operation.
#
#
#   REVISED:
#      $dbc_logname    $out_daytime
#
##########################################################################
# The data is organized in the format separated by |
# The fields in the database are in the following order:
#
#  LIBRF           - Unique record identifier
#                     (Library Reference)
#  OPERTYPE        - cutmthd_libref              Cut Method Library Reference
#  PARTMAT         - part_material_libref Part   Part Material Library Reference
#  TOOLMAT         - tool_material_libref        Tool Material Library Reference
#  DIAM_IN         - cutter diameter - inch
#  DIAM_MM         - cutter diameter - mm
#  LENGTH_IN       - cutter length - inch 
#  LENGTH_MM       - cutter length - mm 
#  DPT_CUT_IN      - dpth_of_cut                 Depth_of_cut(inch)
#  DPT_CUT_MM      - dpth_of_cut                 Depth_of_cut(mm)
#  STEPOVER_IN     - stepover distance (inch)
#  STEPOVER_MM     - stepover distance (mm)
#  SURF_SPEED_FPM  - surface_speed               Suface Speed(FPM)
#  SURF_SPEED_MPM  - surface_speed               Suface Speed(MPM)
#  FEED_IPT        - feed_per_tooth              Feed per Tooth(IPT)
#  FEED_MMPT       - feed_per_tooth              Feed per Tooth(MMPT)
#  APPROACH_PCT    - approach feedrate percentage of cut feedrate
#  ENGAGE_PCT      - engage feedrate percentage of cut feedrate
#  FRSTCUT_PCT     - first cut feedrate percentage of cut feedrate
#  STEPOVER_PCT    - stepover feedrate percentage of cut feedrate
#  RETRACT_PCT     - retract feedrate percentage of cut feedrate
#  RETURN_PCT      - return feedrate percentage of cut feedrate
#  DEPART_PCT      - departure feedrate percentage of cut feedrate
#--------------------------------------------------------------------------------"

    puts $fileid "$headerMessage"

#  load the database structure
    DBC_load_rset_data


#  output FORMAT line
    set format_line "FORMAT"
    for {set i 0} {$i < $dbc_attr_count} {incr i} \
    {
        DBC_ask_attr_id $dbc_attr_aliases($i)
        set format_line "$format_line $dbc_attr_id"
    }
    puts $fileid $format_line
    puts $fileid "#--------------------------------------------------------------------------------"
}
