//////////////////////////////////////////////////////////////////////////////
//
//  imexport.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

//  Include files
#include <stdio.h>
#include <string.h>
#include <io.h>

#include <uf.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>

#include <afxdlgs.h>
#include <Shlobj.h>
//#include <Shlwapi.h>

#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
#   include <iostream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
	using std::cerr;
#else
#	include <strstream.h>
#   include <iostream.h>
#endif

#include "imexport.h"

/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t R_cbs */
#define R_CB_COUNT ( 11 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with R_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t R_cbs[R_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, R_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, R_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_BACK_CB         , 0, R_back_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, R_cancel_cb},
 {R_TOGGLE_TOOLS        , UF_STYLER_VALUE_CHANGED_CB, 0, R_tools_cb},
 {R_TOGGLE_MATERIAL_TOOL, UF_STYLER_VALUE_CHANGED_CB, 0, R_material_tool_cb},
 {R_TOGGLE_FEEDS        , UF_STYLER_VALUE_CHANGED_CB, 0, R_feeds_cb},
 {R_TOGGLE_CUTS         , UF_STYLER_VALUE_CHANGED_CB, 0, R_cuts_cb},
 {R_TOGGLE_STOCKS       , UF_STYLER_VALUE_CHANGED_CB, 0, R_stocks_cb},
 {R_WIDE_DIR            , UF_STYLER_ACTIVATE_CB     , 0, R_dir_cb},
 {R_ACTION_FOLDER       , UF_STYLER_ACTIVATE_CB     , 0, R_folder_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};

//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    char fail_message[133];
    int  response   = 0;
    int  error_code = 0;

    if ( ( UF_initialize() ) != 0 ) {
        MessageBox(NULL, (LPCSTR)"  UG   -", "  ...", MB_ICONSTOP|MB_OK);
    	*returnCode=-1;
    	return;
    }

    mode_export_import=rlen;
    if (mode_export_import<0) mode_export_import=0; //  ()
    if (mode_export_import>1) mode_export_import=1; //  ()

    *returnCode=0;
    // "imexport.dlg"
    if ( ( error_code = UF_STYLER_create_dialog ( "imexport.dlg",
           R_cbs,      /* Callbacks from dialog */
           R_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          uc1601 ("   .dlg - imexport.dlg\n    ", 1 );
          /* Get the user function fail message based on the fail code.*/
          UF_get_fail_message(error_code, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );

          *returnCode=-2;
    }

    UF_terminate();
    return;
}


//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern "C" int ufusr_ask_unload( void )
{
     /* unload immediately after application exits*/
     return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     /*return ( UF_UNLOAD_SEL_DIALOG );*/
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        // Construct a buffer to hold the text.
        ostrstream error_message;

        // Initialize the buffer with the required text.
        error_message << endl
                      << "Error:" << endl
                      << message
                      << endl << endl << ends;

	    // Write the message to standard error
        cerr << error_message.str();
    }
}



/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: R_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    char csCurDir[300];

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     // Make the windows directory the current directory.

     csCurDir[0]='\0';
     GetCurrentDirectory(MAX_PATH, csCurDir);

     data.item_id=R_WIDE_DIR;
     data.item_attr=UF_STYLER_VALUE;
     data.value.string=csCurDir;
     irc=UF_STYLER_set_value(dialog_id,&data);

     csCurDir[0]='\0';
     data.item_id=UF_STYLER_DIALOG_INDEX; // dialog_id; //
     data.item_attr=UF_STYLER_LABEL;
     //  ()
     if (mode_export_import==0) {
     	sprintf(csCurDir,"  ()...");
     }
     //  ()
     if (mode_export_import==1) {
     	sprintf(csCurDir,"  ()...");
     }
     data.value.string=csCurDir;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=UF_STYLER_NAV_INDEX;
     data.item_attr=UF_STYLER_SENSITIVITY;
     data.subitem_index=UF_STYLER_CANCEL_INDEX;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     UF_STYLER_free_value (&data) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}

/* -------------------------------------------------------------------------
 * Callback Name: R_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    char csCurDir[255];
    char *ptr1,*ptr2;
    char tmp[255], tmpCurDir[255], tmpcopy[255];
    char info[5*20+100];
    char mes[255];
    char *fname = "_LIB_XXXXXX";
    char fnametempl[80];
    char lib_names_dat[][20]={
  	"tool_database.dat" ,
  	"cut_methods.dat" ,
  	"feeds_speeds.dat" ,
  	"part_materials.dat" ,
  	"tool_materials.dat" } ;
    int lib_dat[5];
    int i , ans;
    int fl ; //   
    FILE *rf;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

// {UGII_CAM_LIBRARY_TOOL_METRIC_DIR}tool_database.dat
// {UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR}\
//    cut_methods.dat
//    feeds_speeds.dat
//    part_materials.dat
//    tool_materials.dat

/*     */
  UF_translate_variable("UGII_CAM_LIBRARY_TOOL_METRIC_DIR", &ptr1);
  if (ptr1==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_TOOL_METRIC_DIR -  \n : 0 ", 1 );
     UF_terminate ();
     return (UF_UI_CB_CONTINUE_DIALOG);
   }

  UF_translate_variable("UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR", &ptr2);
  if (ptr2==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR -  \n : 0 ", 1 );
     UF_terminate ();
     return (UF_UI_CB_CONTINUE_DIALOG);
   }

     data.item_id=R_WIDE_DIR;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);

     csCurDir[0]='\0';
     strcpy(csCurDir,data.value.string);

  for(i=0;i<5;i++) lib_dat[i]=0;

  data.item_id=R_TOGGLE_TOOLS;
  data.item_attr=UF_STYLER_INTEGER_VALUE;
  irc=UF_STYLER_ask_value(dialog_id,&data);
  if (irc==0) lib_dat[0]=data.value.integer;

  data.item_id=R_TOGGLE_CUTS;
  data.item_attr=UF_STYLER_INTEGER_VALUE;
  irc=UF_STYLER_ask_value(dialog_id,&data);
  if (irc==0) lib_dat[1]=data.value.integer;

  data.item_id=R_TOGGLE_FEEDS;
  data.item_attr=UF_STYLER_INTEGER_VALUE;
  irc=UF_STYLER_ask_value(dialog_id,&data);
  if (irc==0) lib_dat[2]=data.value.integer;

  data.item_id=R_TOGGLE_STOCKS;
  data.item_attr=UF_STYLER_INTEGER_VALUE;
  irc=UF_STYLER_ask_value(dialog_id,&data);
  if (irc==0) lib_dat[3]=data.value.integer;

  data.item_id=R_TOGGLE_MATERIAL_TOOL;
  data.item_attr=UF_STYLER_INTEGER_VALUE;
  irc=UF_STYLER_ask_value(dialog_id,&data);
  if (irc==0) lib_dat[4]=data.value.integer;

  UF_STYLER_free_value (&data) ;

/*            */

  fl=0;
  for(i=0;i<5;i++) if (lib_dat[i]==0) fl++;

  if (fl==5) {
     uc1601 ("     ",1);
     UF_terminate ();
     return (UF_UI_CB_CONTINUE_DIALOG);
  }

  fnametempl[0]='\0';
  strcpy(fnametempl,_mktemp(fname));

  fl=0;
  info[0]='\0'; sprintf(info,"\n");
  for (i=0;i<5;i++)
  {
    tmpcopy[0]='\0';

    /*UGS + */
    tmp[0]='\0';
    if (i==0) strcpy(tmp,ptr1); else strcpy(tmp,ptr2);
    strcpy(tmpcopy,tmp); strcat(tmpcopy,fnametempl);  strcat(tmpcopy,lib_names_dat[i]);
    strcat(tmp,lib_names_dat[i]);

    /* + */
    tmpCurDir[0]='\0';
    strcpy(tmpCurDir,csCurDir);
    strcat(tmpCurDir,"\\");
    strcat(tmpCurDir,lib_names_dat[i]);

    if (lib_dat[i]==1) {

       strcat(info,lib_names_dat[i]);

       //  ()
       if (mode_export_import==0) {

         //PathFileExists(tmp);
         rf=fopen(tmp,"r");
         if (rf==NULL) {
            mes[0]='\0';
            sprintf(mes,"= %s \n   T ?!\n    ...",tmp);
            MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONEXCLAMATION|MB_OK);
            strcat(info," - No\n"); continue ;
         }
         fclose(rf);

         rf=fopen(tmpCurDir,"r");
         if (rf!=NULL) {
            fclose(rf);
            mes[0]='\0';
            sprintf(mes,"= %s \n   !\n   ?",tmpCurDir);
            ans=MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_YESNOCANCEL|MB_DEFBUTTON1);
            if (IDCANCEL==ans) {
            	UF_terminate ();
            	return (UF_UI_CB_CONTINUE_DIALOG);
             }
            if (IDNO==ans) { strcat(info," - No\n"); continue ; }
            // ( DeleteFile(tmpCurDir) == 0)    (remove(tmpCurDir)== -1)
            if (DeleteFile(tmpCurDir) == 0) {
              mes[0]='\0';
              sprintf(mes,"= %s \n   ! ",tmpCurDir);
              MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_OK);
              strcat(info," - No\n"); continue ;
            }
         }
         ans=CopyFile( (LPCTSTR) tmp, (LPCTSTR) tmpCurDir, 0 );
         if (ans==0) {
           mes[0]='\0';
           sprintf(mes,"= %s \n   ! ",tmp);
           MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_OK);
           strcat(info," - No\n"); continue ;
         }

         fl++;
         strcat(info," - Yes\n");

       } //if (mode_export_import==0)

       //  ()
       if (mode_export_import==1) {

         //PathFileExists(tmpCurDir);
         rf=fopen(tmpCurDir,"r");
         if (rf==NULL) {
            mes[0]='\0';
            sprintf(mes,"= %s \n   T ?!\n    ...",tmpCurDir);
            MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONEXCLAMATION|MB_OK);
            strcat(info," - No\n"); continue ;
         }
         fclose(rf);

         rf=fopen(tmp,"r");
         if (rf!=NULL) {
            fclose(rf);
            mes[0]='\0';
            sprintf(mes,"= %s \n  !\n   ?",tmp);
            ans=MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_YESNOCANCEL|MB_DEFBUTTON1);
            if (IDCANCEL==ans) {
            	UF_terminate ();
            	return (UF_UI_CB_CONTINUE_DIALOG);
             }
            if (IDNO==ans) { strcat(info," - No\n"); continue ; }
            ans=CopyFile( (LPCTSTR) tmp , (LPCTSTR) tmpcopy, 0 );
            if (ans==0) {
               mes[0]='\0';
               sprintf(mes,"    = %s ",tmp);
               MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_OK);
            }
         }
         ans=CopyFile( (LPCTSTR) tmpCurDir , (LPCTSTR) tmp, 0 );
         if (ans==0) {
           mes[0]='\0';
           sprintf(mes,"= %s \n   !\n    ",tmpCurDir);
           MessageBox(NULL, (LPCSTR)mes, "  ...", MB_ICONSTOP|MB_OK);
           strcat(info," - No\n"); continue ;
         }

         fl++;
         strcat(info," - Yes\n");


       }

     } //if (lib_dat

  } /*for*/

  if (fl==0) {
    MessageBox(NULL, (LPCSTR)": 0  ", "  ...", MB_ICONEXCLAMATION|MB_OK);
  } else {
   mes[0]='\0';sprintf(mes,"\n   : %d ",fl);
   strcat(info,mes);
   MessageBox(NULL, (LPCSTR)info, "  ...", MB_ICONINFORMATION|MB_OK);
  }

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: R_back_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_back_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog.            */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a back call   */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return (UF_UI_CB_EXIT_DIALOG);


}


/* -------------------------------------------------------------------------
 * Callback Name: R_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: R_tools_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_tools_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_material_tool_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_material_tool_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_feeds_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_feeds_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_cuts_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_cuts_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_stocks_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_stocks_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_dir_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_dir_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: R_folder_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int R_folder_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;

    char csCurDir[MAX_PATH];
    LPSTR pszDisplayName;
    LPTSTR pszPath;
    LPITEMIDLIST pidlRoot = NULL;
    LPITEMIDLIST pidlSelected = NULL;
    BROWSEINFO bi = {0};
    LPMALLOC pMalloc = NULL;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

    csCurDir[0]='\0';
    pszDisplayName=csCurDir;

    SHGetMalloc(&pMalloc);

    //bi.hwndOwner = this;
    bi.pidlRoot = pidlRoot;
    bi.pszDisplayName = pszDisplayName;

    if (mode_export_import==0)
    { bi.lpszTitle = "      ( )...";
      bi.ulFlags = BIF_STATUSTEXT|BIF_EDITBOX;
    } else {
     bi.lpszTitle = " ,      ..";
     bi.ulFlags = BIF_STATUSTEXT;
    }

    bi.lpfn = NULL;
    bi.lParam = 0;

    pidlSelected = SHBrowseForFolder(&bi);

    if(pidlRoot)
    {
        pMalloc->Free(pidlRoot);
    }
    pMalloc->Release();

    if (pidlSelected==NULL) {
       UF_terminate ();
       /* Callback acknowledged, do not terminate dialog */
       return (UF_UI_CB_CONTINUE_DIALOG);
    }

   //AfxMessageBox((LPSTR)pszDisplayName);

   csCurDir[0]='\0';
   pszPath=csCurDir;
   if (TRUE==SHGetPathFromIDList(pidlSelected,pszPath)) {
     //AfxMessageBox(pszPath);
     data.item_id=R_WIDE_DIR;
     data.item_attr=UF_STYLER_VALUE;
     data.value.string=pszPath;
     irc=UF_STYLER_set_value(dialog_id,&data);
     UF_STYLER_free_value (&data) ;
   }

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}






