//////////////////////////////////////////////////////////////////////////////
//
//  libcuts.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

//  Include files
#include <stdio.h>
#include <uf_defs.h>
#include <uf_styler.h>
#include <uf_mb.h>
#include <uf.h>
#include <uf_exit.h>
#include <uf_ui.h>

#include "afxtempl.h"

#include <windows.h>

#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
#   include <iostream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
	using std::cerr;
#else
#	include <strstream.h>
#   include <iostream.h>
#endif

/* These include files are needed for the following template code.            */
#include "libcuts.h"


#define mStr  255

class MATERIALS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; //code         - Unique record identifier
	char MODE[20]; // - Mode Name  (appears on the label)
	char NAME[50]; // - Name  (appears on the label)
	char DESCRIPTION[133]; //  - Description

 MATERIALS ()
 {
   Init();
 }

 virtual ~MATERIALS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MODE[0]='\0'; NAME[0]='\0'; DESCRIPTION[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

 void FormatList(char * str)
 { sprintf(str,"%.20s | %.20s  = %.50s  = %.80s",LIBREF , MODE, NAME , DESCRIPTION );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.20s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MODE, NAME , DESCRIPTION );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.20s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MODE, NAME , DESCRIPTION );  }

} ;

 CArray < MATERIALS, MATERIALS& > myTM ;

 char cut_methods[255];
 int SaveFlag;


/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t CM_cbs */
#define CM_CB_COUNT ( 5 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with CM_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t CM_cbs[CM_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, CM_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, CM_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, CM_cancel_cb},
 {CM_LIST_CUTS          , UF_STYLER_ACTIVATE_CB     , 0, CM_list_cuts_cb},
 {CM_BLO_ACT            , UF_STYLER_ACTIVATE_CB     , 1, CM_blo_act_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};

//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int i ;
    int response=0;
    char fail_message[133];
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
     i=Material_Init();
     if (i<0) {
        fail_message[0]='\0';
        sprintf(fail_message,"        \n    .");
        MessageBeep(0);
        MessageBox(GetFocus(),(LPCTSTR)fail_message," - ->.",MB_ICONSTOP|MB_OK);
     }

    if ( ( errorCode = UF_STYLER_create_dialog ( "libcuts.dlg",
           CM_cbs,      /* Callbacks from dialog */
           CM_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          /* Get the user function fail message based on the fail code.*/
          fail_message[0]='\0';
          UF_get_fail_message(errorCode, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );
          uc1601("   'libcuts.dlg'  ...",TRUE);
    }


        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode=0;
}

//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern "C" int ufusr_ask_unload( void )
{
     /* unload immediately after application exits*/
     return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     //return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}


/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}


/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        // Construct a buffer to hold the text.
        ostrstream error_message;

        // Initialize the buffer with the required text.
        error_message << endl
                      << "Error:" << endl
                      << message
                      << endl << endl << ends;

	    // Write the message to standard error
        cerr << error_message.str();
    }
}


/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: CM_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int CM_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    long int i , l;
    char str[255] ; // CString
    //char ls[255];
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     i=Material_Load();
     if (i==0) {
       l= myTM.GetSize();
       if (l>0) {
	 data.item_id=CM_LIST_CUTS;
         data.item_attr=UF_STYLER_LIST_INSERT;
         data.subitem_index=UF_STYLER_NO_SUB_INDEX;
         data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
         for (i=0;i<l;i++) {
           tm = myTM.GetAt(i);
           str[0]='\0'; tm.FormatList(str);
           //MessageBox(GetFocus(),(LPCTSTR)str,"TM_construct_cb",MB_ICONSTOP|MB_OK);
           data.subitem_index=i;
	   data.value.string=str; //(char**)ls;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=0;
         irc=UF_STYLER_set_value(dialog_id,&data);
         UF_STYLER_free_value (&data) ;
       }
     }

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: CM_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int CM_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: CM_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int CM_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: CM_list_cuts_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int CM_list_cuts_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: CM_blo_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int CM_blo_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i , k , l;
    char str[255] ;
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=CM_BLO_ACT;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     i=data.value.integer;

  data.item_id=CM_LIST_CUTS;

  switch (i) {
   case 0 : // 

         data.item_attr=UF_STYLER_VALUE;
         data.indicator=UF_STYLER_INTEGER_VALUE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         k=data.value.integer ;
         irc=Material_Edit_New(k);
         if (k>=0 && irc==UF_UI_OK) {

            data.item_attr=UF_STYLER_LIST_DELETE;
            data.subitem_index=UF_STYLER_NO_SUB_INDEX;
            irc=UF_STYLER_set_value(dialog_id,&data);

            data.item_attr=UF_STYLER_LIST_INSERT;
            data.indicator=UF_STYLER_STRING_VALUE ;
            for (i=0;i<myTM.GetSize();i++) {
              tm = myTM.GetAt(i);
              str[0]='\0'; tm.FormatList(str);
              data.subitem_index=i;
              data.value.string=str;
              irc=UF_STYLER_set_value(dialog_id,&data);
            }

            data.item_attr=UF_STYLER_VALUE;
            data.subitem_index=k;
            irc=UF_STYLER_set_value(dialog_id,&data);

         }
   break;
   case 1 : //  

         data.item_attr=UF_STYLER_VALUE;
         data.indicator=UF_STYLER_INTEGER_VALUE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         k=data.value.integer;

         i=Material_Delete(k);
         if (i<0) break ;

         l= myTM.GetSize();
         if (l<0) break ;

         data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         for (i=0;i<l;i++) {
           tm = myTM.GetAt(i);
           str[0]='\0'; tm.FormatList(str);
           data.item_attr=UF_STYLER_LIST_INSERT;
           data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }

         if (k<=1) k=0; else k--;
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=k;
         irc=UF_STYLER_set_value(dialog_id,&data);
   break;
   case 2 : //  
         irc=Material_Edit_New(-1);
         if (irc==UF_UI_OK) {

            data.item_attr=UF_STYLER_LIST_DELETE;
            data.subitem_index=UF_STYLER_NO_SUB_INDEX;
            irc=UF_STYLER_set_value(dialog_id,&data);

            data.item_attr=UF_STYLER_LIST_INSERT;
            data.indicator=UF_STYLER_STRING_VALUE ;
            for (i=0;i<myTM.GetSize();i++) {
              tm = myTM.GetAt(i);
              str[0]='\0'; tm.FormatList(str);
              data.subitem_index=i;
              data.value.string=str;
              irc=UF_STYLER_set_value(dialog_id,&data);
            }

            data.item_attr=UF_STYLER_VALUE;
            data.subitem_index=myTM.GetSize()-1;
            irc=UF_STYLER_set_value(dialog_id,&data);

         }

   break;
   case 3 : // 
         Material_List_Help();
   break;
   case 4 : //  

	 data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         i=Material_Load();
         if (i<0) break ;

         l= myTM.GetSize();
         if (l<=0) break ;

         data.item_attr=UF_STYLER_LIST_INSERT;
         data.indicator=UF_STYLER_STRING_VALUE ;
         for (i=0;i<l;i++) {
           tm = myTM.ElementAt(i);
           str[0]='\0';tm.FormatList(str);
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=0;
         irc=UF_STYLER_set_value(dialog_id,&data);
   break;
   case 5 : //  
         Material_Save();
   break;

   case 6 : break;
   default : uc1601(" ",TRUE);
  }

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}

// Material.cpp: implementation of the Material class.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

/*
**********************************************************
**********************************************************
*/
int Material_Init()
{
/*    UGS -  */
// {UGII_CAM_LIBRARY_TOOL_METRIC_DIR}tool_database.dat
// {UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR}\
//    cut_methods.dat
//    feeds_speeds.dat
//    part_materials.dat
//    tool_materials.dat

  char *ptr1;
  int fl ;
  FILE *rf;

  cut_methods[0]='\0';
  SaveFlag=0;

  UF_translate_variable("UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR", &ptr1);
  if (ptr1==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR -  ", 1 );
     return(-1);
   }

/*          */
  cut_methods[0]='\0';
  strcpy(cut_methods,ptr1); strcat(cut_methods,"cut_methods.dat");

  fl=0;
  rf=fopen(cut_methods,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  rf=fopen(cut_methods,"a");
  if (rf==NULL) { SaveFlag=-1; } else { fclose(rf); }

  if (fl!=0) {
   return(-2);
  }

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 MATERIALS tm;

 rf=fopen(cut_methods,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 //myTM.SetSize(50);
 try
 {
    myTM.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MODE,stmp,k); //strcat(tm.MODE,"\0");
	   tm.MODE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.NAME,stmp,k); //strcat(tm.NAME,"\0");
	   tm.NAME[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.DESCRIPTION,stmp,k);
	   } else {
	     strcpy(tm.DESCRIPTION,stmp);
	   }
           //strcat(tm.DESCRIPTION,"\0");
           tm.DESCRIPTION[k]='\0';

           myTM.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Delete(int index)
{
 if (index<0 || index>=myTM.GetSize()) return (-1);

 try
 {
   myTM.RemoveAt(index);
 }
  catch( char * str )
  {
    cerr << "Exception raised: " << str << '\n';
  }

 return (0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Save()
{
 int index , l;
 char str[mStr] ;
 FILE *rf;
 MATERIALS tm;

 if (SaveFlag==-1) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 l=myTM.GetSize();
 if (l<=0) {
   uc1601("   \n  ...",TRUE);
   return(-1);
  }

 index=MessageBox(GetFocus(),(LPCTSTR)" ,      ?\n       -   \n   ..."," ...",MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
 if (index==IDNO) return (0) ;

 rf=fopen(cut_methods,"w");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-2);
 }

 fputs("# CUT_METHODS.DAT\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("#\n",rf);
 fputs("#   PURPOSE:\n",rf);
 fputs("#\n",rf);
 fputs("#      This is the database file used for defining \"Cut Method\" in\n",rf);
 fputs("#      Machining method.  The library reference is used for Feeds and\n",rf);
 fputs("#      Speeds Specifications.\n",rf);
 fputs("#\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("##\n",rf);
 fputs("##  The following key words for Attribute ids are defined\n",rf);
 fputs("##\n",rf);
 fputs("##  LIBREF   cutmthd_libref   - Unique record identifier (Library Reference)\n",rf);
 fputs("##                                   (Library Reference)\n",rf);
 fputs("##  MODE     machine_mode     - Machine mode - MILL\n",rf);
 fputs("##                                           - DRILL\n",rf);
 fputs("##                                           - LATHE\n",rf);
 fputs("##  NAME     name             - Name of Cutting Method \n",rf);
 fputs("##                              (appears on the label in method view)\n",rf);
 fputs("#########################################################################\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);
 fputs("FORMAT LIBRF MODE NAME DESCRIPTION\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);

 l=myTM.GetSize();
 for (index=0;index<l;index++)
 {
   tm = myTM.GetAt(index);
   str[0]='\0'; tm.FormatFile(str);
   fputs(str,rf);
 }

 fclose(rf);

 str[0]='\0'; sprintf(str," ...\n   %ld - .", l );
 MessageBox(GetFocus(),(LPCTSTR)str," ...",MB_ICONINFORMATION|MB_OK);

 return(0);
}



/*
**********************************************************
**********************************************************
*/
int Material_List_Help()
{
  int index ;
  char mes[mStr] ;
  char *release;
  MATERIALS tm;

  UF_UI_open_listing_window();

  UF_get_release (&release );
  mes[0]='\0'; sprintf(mes,"\n  Unigraphics - %s ",release); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n   ."); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n\t . (Win32 API, MFC, VC++6.0) "); UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n      :     .");
  UF_UI_write_listing_window("\n#============================================================");

  UF_UI_write_listing_window("\n    :");
  UF_UI_write_listing_window("\n  '' -   ");
  UF_UI_write_listing_window("\n  ' ' -   ");
  UF_UI_write_listing_window("\n  ' ' -   ,     ");
  UF_UI_write_listing_window("\n  '/' -       .");
  UF_UI_write_listing_window("\n  ' ' -     ");
  UF_UI_write_listing_window("\n  ' ' -     ");

  UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

UF_UI_write_listing_window("\n\n\t  Accessing the Cut Method Library [    ]");
UF_UI_write_listing_window("\n          Cut Method [ ].");
UF_UI_write_listing_window("\n        :");
UF_UI_write_listing_window("\n       MB3      ,     ");
UF_UI_write_listing_window("\n              Insert [] -> Method [].");
UF_UI_write_listing_window("\n       Insert [] -> Method []      ");
UF_UI_write_listing_window("\n      Create Method [ ].");
UF_UI_write_listing_window("\n       Insert Method [ ]");
UF_UI_write_listing_window("\n      ");
UF_UI_write_listing_window("\n       1.  OK      Method [].");
UF_UI_write_listing_window("\n       2.   Cut Method [ ]    Search Result [ ]");
UF_UI_write_listing_window("\n             .");
UF_UI_write_listing_window("\n       3.      Cut Method Library [  ].");

UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n        'Cut Methods'.");
  UF_UI_write_listing_window("\n      :");
  UF_UI_write_listing_window("\n\t  LIBREF   -    (Library Reference)");
  UF_UI_write_listing_window("\n\t  MODE     -   { MILL , DRILL , LATHE }");
  UF_UI_write_listing_window("\n\t  NAME     -   ");
  UF_UI_write_listing_window("\n ");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myTM.GetSize(), SaveFlag , cut_methods);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT LIBRF MODE NAME DESCRIPTION");
  UF_UI_write_listing_window("\n#-----------------------------------------------------------\n");
  for (index=0;index<myTM.GetSize();index++)
  {
    tm = myTM.GetAt(index);
    mes[0]='\0';  tm.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n ");
  UF_UI_write_listing_window("\n#============================================================\n$$");

  //UF_UI_close_listing_window () ;

 return(0);

}


/*
**********************************************************
**********************************************************
*/
char * Material_NewInd()
{
   int index ;
   long int i;
   char ls[25], *ptr ;
   MATERIALS tm;

   /* Seed the random-number generator with current time so that
    * the numbers will be different every time we run. */
  srand( (unsigned)time( NULL ) );
  ls[0]='\0'; sprintf(ls,"OPD0_%04d",rand() );

  for (i=0,index=0;index<myTM.GetSize();index++)
  {
    tm = myTM.GetAt(index);
    if (0==strcmp( ls, tm.LIBREF)) {
    	if (i>1000) break;
    	ls[0]='\0'; sprintf(ls,"OPD0_%04d",rand() );
    	i++;
    	continue;
     }
  }

  ptr = (char *) malloc(strlen(ls)+5);
  strcpy(ptr,ls);strcat(ptr,"\0");
 return(ptr);
}




//==================================================================

/************************************************************************************************/

int INDEX_REC ; // if INDEX_REC>=0   
                // if INDEX_REC=-1  

#include "libcutsedit.h"

/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t EC_cbs */
#define EC_CB_COUNT ( 7 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with EC_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t EC_cbs[EC_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, EC_cons_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, EC_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, EC_cancel_cb},
 {EC_WIDE_NUM           , UF_STYLER_ACTIVATE_CB     , 0, EC_cancel_cb},
 {EC_MODE_CUTS          , UF_STYLER_ACTIVATE_CB     , 0, EC_mode_cuts},
 {EC_WIDE_MAT           , UF_STYLER_ACTIVATE_CB     , 0, EC_name_cb},
 {EC_WIDE_DISCRIPTION   , UF_STYLER_ACTIVATE_CB     , 0, EC_description_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};


int Material_Edit_New(int index)
{
    char fail_message[133];
    int  response   = 0;
    int  error_code = 0;

    if ( ( UF_initialize() ) != 0 ) return (-1);

    INDEX_REC=index;

    if ( ( error_code = UF_STYLER_create_dialog ( "libcutsedit.dlg",
           EC_cbs,      /* Callbacks from dialog */
           EC_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          /* Get the user function fail message based on the fail code.*/
          UF_get_fail_message(error_code, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );
          uc1601("   'libcutsedit.dlg'  ...",TRUE);
          UF_terminate();
          return (-2);
    }

    UF_terminate();
    return (response);
}


/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: EC_cons_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_cons_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    int i ;
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    MATERIALS tm;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0) return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     data.item_attr=UF_STYLER_VALUE ;

     if (INDEX_REC>=0) {
       tm = myTM.GetAt(INDEX_REC);

       data.item_id=EC_WIDE_NUM ;
       data.value.string=tm.LIBREF;
       irc=UF_STYLER_set_value(dialog_id,&data);

       i=0;
       if (0==strcmp( strupr(tm.MODE), "DRILL")) { i=1; }
       if (0==strcmp( strupr(tm.MODE), "LATHE")) { i=2; }

       data.item_id=EC_MODE_CUTS;
       data.value.integer=i;
       data.subitem_index=i;
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_id=EC_WIDE_MAT ;
       data.value.string=tm.NAME;
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_id=EC_WIDE_DISCRIPTION ;
       data.value.string=tm.DESCRIPTION;
       irc=UF_STYLER_set_value(dialog_id,&data);

     } else {

       data.item_id=EC_WIDE_NUM ;
       data.value.string=Material_NewInd();
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_id=EC_MODE_CUTS ;
       data.subitem_index=0 ; // "MILL\0";
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_id=EC_WIDE_MAT ;
       data.value.string="-\0";
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_id=EC_WIDE_DISCRIPTION ;
       data.value.string="0\0";
       irc=UF_STYLER_set_value(dialog_id,&data);

     }

    data.item_attr=UF_STYLER_FOCUS ;

    data.item_id=EC_WIDE_MAT ;
    irc=UF_STYLER_set_value(dialog_id,&data);

    UF_STYLER_free_value (&data) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: EC_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    int i ;
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    MATERIALS tm;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0) return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_attr=UF_STYLER_VALUE ;

     if (INDEX_REC>=0) {

       tm = myTM.GetAt(INDEX_REC);

       data.item_id=EC_WIDE_NUM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.LIBREF,data.value.string);
       strcat(tm.LIBREF,"\0");

       data.item_id=EC_MODE_CUTS ;
       //data.indicator=UF_STYLER_INTEGER_VALUE;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       i=data.value.integer;
       switch (i) {
       	case 1: strcpy(tm.MODE,"DRILL\0"); break ;
       	case 2: strcpy(tm.MODE,"LATHE\0"); break ;
       	default : strcpy(tm.MODE,"MILL\0"); break ;
       	}

       data.item_id=EC_WIDE_MAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.NAME,data.value.string);
       strcat(tm.NAME,"\0");

       data.item_id=EC_WIDE_DISCRIPTION ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.DESCRIPTION,data.value.string);
       strcat(tm.DESCRIPTION,"\0");

       myTM.SetAt(INDEX_REC, tm);

     } else {

       tm.Init();

       strcpy(tm.FORMAT,"DATA\0");

       data.item_id=EC_WIDE_NUM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.LIBREF,data.value.string);
       strcat(tm.LIBREF,"\0");

       data.item_id=EC_MODE_CUTS ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       i=data.value.integer;
       switch (i) {
       	case 1: strcpy(tm.MODE,"DRILL\0"); break ;
       	case 2: strcpy(tm.MODE,"LATHE\0"); break ;
       	default : strcpy(tm.MODE,"MILL\0"); break ;
       	}

       data.item_id=EC_WIDE_MAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.NAME,data.value.string);
       strcat(tm.NAME,"\0");

       data.item_id=EC_WIDE_DISCRIPTION ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(tm.DESCRIPTION,data.value.string);
       strcat(tm.DESCRIPTION,"\0");

       myTM.Add( tm );

      }

     UF_STYLER_free_value (&data) ;

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: EC_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: EC_mode_cuts
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_mode_cuts ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: EC_name_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_name_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: EC_description_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int EC_description_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}

