//////////////////////////////////////////////////////////////////////////////
//
//  libfeeds.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

//  Include files
#include <stdio.h>
#include <malloc.h>

#include <uf_defs.h>
#include <uf.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>
#include <uf_mb.h>

#include "afxtempl.h"

#include <windows.h>

#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
#   include <iostream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
	using std::cerr;
#else
#	include <strstream.h>
#   include <iostream.h>
#endif

/* These include files are needed for the following template code.            */
#include "libfeeds.h"

#define mStr  255

class FEEDS
{
 public :

	char FORMAT[10];
	char LIBREF[25]; // - Unique record identifier (Library Reference)
	char OPERTYPE[20];//- cutmthd_libref              Cut Method Library Reference
	char PARTMAT[20]; //- part_material_libref Part   Part Material Library Reference
	char TOOLMAT[20]; //- tool_material_libref        Tool Material Library Reference
	double DPT_CUT_IN; //- dpth_of_cut                 Depth_of_cut(inch)
	double DPT_CUT_MM; //- dpth_of_cut                 Depth_of_cut(mm)
	double INDEX1 ; //
	double INDEX2 ; //
	double SURF_SPEED_FPM; //- surface_speed               Suface Speed(FPM)
	double SURF_SPEED_MPM; //- surface_speed               Suface Speed(MPM)
	double FEED_IPT; //- feed_per_tooth              Feed per Tooth(IPT)
	double FEED_MMPT; // - feed_per_tooth              Feed per Tooth(MMPT)

 FEEDS ()
 {
   Init();
 }

 virtual ~FEEDS () { ; }

 void Init() {
   FORMAT[0]='\0'; LIBREF[0]='\0'; OPERTYPE[0]='\0'; PARTMAT[0]='\0'; TOOLMAT[0]='\0';
   DPT_CUT_IN=0.;  DPT_CUT_MM=0.;
   INDEX1=0.;  INDEX2=0.;
   SURF_SPEED_FPM=0.;  SURF_SPEED_MPM=0.;
   FEED_IPT=0.;  FEED_MMPT=0.;
 }

 void FormatList(char * str)
 {
   if (INDEX1!=0 || INDEX2!=0) {
     sprintf(str,"%.20s  =  %.3lf  i1=  %.2lf  i2=  %.2lf =  %.3lf  =  %.3lf ",LIBREF, DPT_CUT_MM, INDEX1 ,INDEX2, SURF_SPEED_MPM , FEED_MMPT );
   } else {
     sprintf(str,"%.20s  =  %.3lf  =  %.3lf  =  %.3lf ",LIBREF, DPT_CUT_MM, SURF_SPEED_MPM , FEED_MMPT );
   }
 }

 void FormatFile(char * str)
 {
   double t1,t2,t3;
   t1=DPT_CUT_MM/25.4;
   t2=calc_surface_speed_inh(SURF_SPEED_MPM);
   t3=calc_feed_per_tooth_inh(FEED_MMPT);

   if (INDEX1!=0 || INDEX2!=0) {
     sprintf(str,"%.5s|%.20s|%.20s|%.20s|%.20s|%.3lf|%.3lf|%.2lf|%.2lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBREF,OPERTYPE,PARTMAT,TOOLMAT, \
                      t1, DPT_CUT_MM, \
                      INDEX1, INDEX2, \
                      t2,SURF_SPEED_MPM , \
                      t3,FEED_MMPT );
     return ;
   }
   sprintf(str,"%.5s|%.20s|%.20s|%.20s|%.20s|%.3lf|%.3lf|||%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBREF,OPERTYPE,PARTMAT,TOOLMAT, \
                      t1, DPT_CUT_MM, \
                      t2,SURF_SPEED_MPM , \
                      t3,FEED_MMPT );
 }

 void FormatHelp(char * str)
 {
   double t1,t2,t3;
   t1=DPT_CUT_MM/25.4;
   t2=calc_surface_speed_inh(SURF_SPEED_MPM);
   t3=calc_feed_per_tooth_inh(FEED_MMPT);

   sprintf(str,"%.5s|%.20s|%.20s|%.20s|%.20s|%.3lf|%.3lf|%.2lf|%.2lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBREF,OPERTYPE,PARTMAT,TOOLMAT, \
                      t1, DPT_CUT_MM, \
                      INDEX1, INDEX2, \
                      t2,SURF_SPEED_MPM , \
                      t3,FEED_MMPT );

  }

/*   # Change the dbc_surface_speed obtained to english
     # speed in english = speed in metric * 1000 / (12 * 25.4)

     set dbc_surface_speed [expr $dbc_surface_speed*1000.0/12.0/25.4 ]
     set dbc_feed_per_tooth [expr $dbc_feed_per_tooth/25.4 ]
     */
 double calc_surface_speed_inh(double v)
 {
  double t; t=v*1000.0/(12.0*25.4) ;
  return(t) ;
 }

 double calc_feed_per_tooth_inh(double v)
 {
  double t; t=v/25.4 ;
  return(t) ;
 }

} ;

 CArray < FEEDS, FEEDS& > myFEEDS ;
 char feeds_speeds[255];
 int SaveFlag;

/*========================================================================================*/

 int change_mode; // edit or new

/*========================================================================================*/
class CUTS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; //code         - Unique record identifier
	char MODE[20]; // - Mode Name  (appears on the label)
	char NAME[50]; // - Name  (appears on the label)
	char DESCRIPTION[133]; //  - Description

 CUTS ()
 {
   Init();
 }

 virtual ~CUTS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MODE[0]='\0'; NAME[0]='\0'; DESCRIPTION[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

 void FormatList(char * str)
 { sprintf(str,"%.20s | %.20s  = %.50s  = %.80s",LIBREF , MODE, NAME , DESCRIPTION );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.20s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MODE, NAME , DESCRIPTION );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.20s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MODE, NAME , DESCRIPTION );  }

} ;

 CArray < CUTS, CUTS& > myCUTS ;
 char cut_methods[255];

/*========================================================================================*/
class TOOL_MATERIALS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; //material_code         - Unique record identifier
	char MATNAM[50]; //material_name         - Material Name  (appears on the label)
	char MATDESC[133]; //material_description  - Material Description

 TOOL_MATERIALS ()
 {
   Init();
 }

 virtual ~TOOL_MATERIALS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MATNAM[0]='\0'; MATDESC[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

 void FormatList(char * str)
 { sprintf(str,"%.50s  = %.80s",MATNAM, MATDESC );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.20s|%.50s|%.133s\n", FORMAT , LIBREF , MATNAM , MATDESC );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MATNAM, MATDESC );  }

} ;

 CArray < TOOL_MATERIALS, TOOL_MATERIALS& > myTM ;
 char tool_materials[255];

/*========================================================================================*/
class PARTS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; // partmaterial_libref    - Unique record identifier
	char MATCODE[50]; //MATCODE   material_code          - Material Code
	char MATNAME[50]; //  MATNAME   material_name          - Material Name (appears on the label)
	char PARTMAT[133]; // PARTMAT   material_description   - Material Description
	char HARDNESS[20]; // HARDNESS  material_hardness      - Material Hardness

 PARTS ()
 {
   Init();
 }

 virtual ~PARTS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MATCODE[0]='\0'; MATNAME[0]='\0'; PARTMAT[0]='\0'; HARDNESS[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

// |||:
 void FormatList(char * str)
 { sprintf(str,"%.50s  =  %.50s  =  %.20s  =  %.80s",MATCODE , MATNAME, HARDNESS , PARTMAT );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.50s|%.50s|%.133s|%.20s|%.20s\n", FORMAT,MATCODE,MATNAME,PARTMAT,HARDNESS,LIBREF );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.50s|%.50s|%.133s|%.20s|%.20s\n", FORMAT,MATCODE,MATNAME,PARTMAT,HARDNESS,LIBREF );  }

} ;

 CArray < PARTS, PARTS& > myPARTS ;
 char part_materials[255];




/*========================================================================================*/

/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t PF_cbs */
#define PF_CB_COUNT ( 16 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with PF_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t PF_cbs[PF_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, PF_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, PF_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, PF_cancel_cb},
 {PF_LIBREF             , UF_STYLER_ACTIVATE_CB     , 0, PF_libref_cb},
 {PF_OPERTYPE           , UF_STYLER_ACTIVATE_CB     , 0, PF_opertype_cb},
 {PF_PARTMAT            , UF_STYLER_ACTIVATE_CB     , 0, PF_partmat_cb},
 {PF_TOOLMAT            , UF_STYLER_ACTIVATE_CB     , 0, PF_toolmat_cb},
 {PF_DPT_CUT_MM         , UF_STYLER_ACTIVATE_CB     , 0, PF_dpt_cut_mm_cb},
 {PF_CALC               , UF_STYLER_ACTIVATE_CB     , 1, PF_action_calc_cb},
 {PF_INDEX1             , UF_STYLER_ACTIVATE_CB     , 0, PF_index1_cb},
 {PF_INDEX2             , UF_STYLER_ACTIVATE_CB     , 0, PF_index2_cb},
 {PF_SURF_SPEED_MPM     , UF_STYLER_ACTIVATE_CB     , 0, PF_surface_speed_cb},
 {PF_FEED_MMPT          , UF_STYLER_ACTIVATE_CB     , 0, PF_feed_per_tooth_cb},
 {PF_BLO_ACTION         , UF_STYLER_ACTIVATE_CB     , 0, PF_edit_act_cb},
 {PF_LIST               , UF_STYLER_ACTIVATE_CB     , 1, PF_list_cb},
 {PF_BLO_ACT            , UF_STYLER_ACTIVATE_CB     , 1, PF_blo_act_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};


/*--------------------------------------------------------------------------
UF_MB_styler_actions_t contains 4 fields.  These are defined as follows:

Field 1 : the name of your dialog that you wish to display.
Field 2 : any client data you wish to pass to your callbacks.
Field 3 : your callback structure.
Field 4 : flag to inform menubar of your dialog location.  This flag MUST
          match the resource set in your dialog!  Do NOT ASSUME that changing
          this field will update the location of your dialog.  Please use the
          UIStyler to indicate the position of your dialog.
--------------------------------------------------------------------------*/
static UF_MB_styler_actions_t actions[] = {
    { "libfeeds.dlg",  NULL,   PF_cbs,  UF_MB_STYLER_IS_NOT_TOP },
    { NULL,  NULL,  NULL,  0 } /* This is a NULL terminated list */
};




//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int i ;
    int response=0;
    char fail_message[133];
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
     i=Inits();
     if (i<0) {
        fail_message[0]='\0';
        sprintf(fail_message," '  '      \n    .");
        MessageBeep(0);
        MessageBox(GetFocus(),(LPCTSTR)fail_message," - ->.",MB_ICONSTOP|MB_OK);
        errorCode = UF_terminate();
        PrintErrorMessage( errorCode );
        *returnCode=0;
        return;
     }

    if ( ( errorCode = UF_STYLER_create_dialog ( "libfeeds.dlg",
           PF_cbs,      /* Callbacks from dialog */
           PF_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          /* Get the user function fail message based on the fail code.*/
          fail_message[0]='\0';
          UF_get_fail_message(errorCode, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );
          uc1601("   'libfeeds.dlg' \n    ?...",TRUE);
    }

        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode=0;
}

//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern int ufusr_ask_unload (void)
{
     /* unload immediately after application exits*/
     return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     //return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}



/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        // Construct a buffer to hold the text.
        ostrstream error_message;

        // Initialize the buffer with the required text.
        error_message << endl
                      << "Error:" << endl
                      << message
                      << endl << endl << ends;

	    // Write the message to standard error
        cerr << error_message.str();
    }
}


/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: PF_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    long int i , l , dim;
    char str[133] ; // CString
    char *ls[500];

  FEEDS  fd;
  TOOL_MATERIALS tm;
  CUTS cts;
  PARTS prts;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     change_mode=-1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;
     data.item_id=PF_SW_EDIT;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     i=Loads();
     if (i==0) {
       l= myFEEDS.GetSize();
       if (l>0) {
	 data.item_id=PF_LIST;
         data.item_attr=UF_STYLER_LIST_INSERT;
         data.subitem_index=UF_STYLER_NO_SUB_INDEX;
         data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
         for (i=0;i<l;i++) {
           fd = myFEEDS.GetAt(i);
           str[0]='\0'; fd.FormatList(str);
           data.subitem_index=i;
	   data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=0;
         irc=UF_STYLER_set_value(dialog_id,&data);

          dim=myCUTS.GetSize();
          for (i=0;i<dim;i++)
          {
            cts = myCUTS.GetAt(i);
            ls[i]= (char *) malloc ((132+1)*sizeof(char)) ;
            ls[i][0]='\0';  cts.FormatList(ls[i]);
          }

          data.item_id=PF_OPERTYPE ;
          data.item_attr=UF_STYLER_SUBITEM_VALUES;
          data.value.strings=ls;
          data.count=dim;
          irc=UF_STYLER_set_value(dialog_id,&data);

          dim=myTM.GetSize();
          for (i=0;i<dim;i++)
          {
            tm = myTM.GetAt(i);
            ls[i]= (char *) malloc ((132+1)*sizeof(char)) ;
            ls[i][0]='\0';  tm.FormatList(ls[i]);
          }

          data.item_id=PF_TOOLMAT ;
          data.item_attr=UF_STYLER_SUBITEM_VALUES;
          data.value.strings=ls;
          data.count=dim;
          irc=UF_STYLER_set_value(dialog_id,&data);

          dim=myPARTS.GetSize();
          for (i=0;i<dim;i++)
          {
            prts = myPARTS.GetAt(i);
            ls[i]= (char *) malloc ((132+1)*sizeof(char)) ;
            ls[i][0]='\0';  prts.FormatList(ls[i]);
          }

          data.item_id=PF_PARTMAT ;
          data.item_attr=UF_STYLER_SUBITEM_VALUES;
          data.value.strings=ls;
          data.count=dim;
          irc=UF_STYLER_set_value(dialog_id,&data);

         fd = myFEEDS.GetAt(0);

         data.item_attr=UF_STYLER_VALUE ;

         data.item_id=PF_LIBREF ;
         data.value.string=fd.LIBREF;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_OPERTYPE ;
         data.subitem_index=FindIndex(1,0);
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_PARTMAT ;
         data.subitem_index=FindIndex(3,0);
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_TOOLMAT ;
         data.subitem_index=FindIndex(2,0);
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_DPT_CUT_MM ;
         data.value.real=fd.DPT_CUT_MM;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_INDEX1 ;
         data.value.real=fd.INDEX1;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_INDEX2 ;
         data.value.real=fd.INDEX2;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_SURF_SPEED_MPM ;
         data.value.real=fd.SURF_SPEED_MPM;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PF_FEED_MMPT ;
         data.value.real=fd.FEED_MMPT;
         irc=UF_STYLER_set_value(dialog_id,&data);

       }
     }

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_libref_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_libref_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_opertype_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_opertype_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_partmat_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_partmat_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_toolmat_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_toolmat_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_dpt_cut_mm_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_dpt_cut_mm_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}



/* -------------------------------------------------------------------------
 * Callback Name: PF_action_calc_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_action_calc_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
  int response ;
  char menu_main[14][38];
  char cp2[9 ][ 16 ];
  int ip5;
  double value_dlg[3];
  int i ;
  double value;
  char mes[133];

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

  strcpy(&menu_main[0][0], ":  \0");
  strcpy(&menu_main[1][0], ":   \0");

  response=3;
  while (response != 1 && response != 2  && response != 19 )
  {

   response = uc1603(" .....",1,menu_main, 1+1);

   if (response >= 3 || response < 19 )
   {
     switch (response)
     {
  	case 5 :/* */

         while (1) {

           for (i=0;i<3;i++) value_dlg[i]=0;

           strcpy(&cp2[0][0], ":. =\0");
           strcpy(&cp2[1][0], ": .=\0");
           //strcpy(&cp2[2][0], ":\0");

           response=0;
           response=uc1609 (" .....",cp2,2,value_dlg, &ip5);
           if (response != 3 && response != 4 ) break ;

           if (value_dlg[0]>=0. && value_dlg[1]>=0.) {
           	value=3.1415926*value_dlg[0]*value_dlg[1]/1000.0;

                data.item_attr=UF_STYLER_VALUE ;
                data.item_id=PF_SURF_SPEED_MPM ;
                data.value.real=value;
                irc=UF_STYLER_set_value(dialog_id,&data);

                mes[0];sprintf(mes,"  = %.3lf (m/)",value);
                uc1601(mes,TRUE);
                response=1;
                break ;

            }

          }//while(1)

  	break ;
  	case 6 :/*  */

         while (1) {

           for (i=0;i<3;i++) value_dlg[i]=0;

           strcpy(&cp2[0][0], ":=\0");
           strcpy(&cp2[1][0], ":N . .=\0");
           strcpy(&cp2[2][0], ": .=\0");

           response=0;
           response=uc1609 (" .....",cp2,3,value_dlg, &ip5);
           if (response != 3 && response != 4 ) break ;

           if (value_dlg[0]>=0. && value_dlg[1]>0. && value_dlg[2]>0.) {
           	value=value_dlg[0]/(value_dlg[1]*value_dlg[2]);

                data.item_attr=UF_STYLER_VALUE ;
                data.item_id=PF_FEED_MMPT ;
                data.value.real=value;
                irc=UF_STYLER_set_value(dialog_id,&data);

                mes[0];sprintf(mes,"   = %.3lf (/)",value);
                uc1601(mes,TRUE);
                response=1;
                break ;

            }

          }//while(1)

  	break ;
       default : break ;
     }
   }

  } // end while uc1603


     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_index1_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_index1_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_index2_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_index2_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_surface_speed_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_surface_speed_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_feed_per_tooth_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_feed_per_tooth_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_edit_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_edit_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    long int i , index;
    char str[mStr] ;
    FEEDS fd;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PF_BLO_ACTION;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     i=data.value.integer;

  switch (i) {
   case 0 : // OK

     if (change_mode==0) // edit or new
     {

       data.item_id=PF_LIST;
       data.item_attr=UF_STYLER_VALUE;
       data.indicator=UF_STYLER_INTEGER_VALUE ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       index=data.value.integer ;

       if (index<0) break ;

       fd = myFEEDS.GetAt(index);

       data.item_attr=UF_STYLER_VALUE ;

       data.item_id=PF_LIBREF ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.LIBREF,data.value.string); strcat(fd.LIBREF,"\0");

       data.indicator=UF_STYLER_INTEGER_VALUE;
       data.count=1;

       data.item_id=PF_OPERTYPE ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.OPERTYPE,RecordIndex(1,data.value.integer)); strcat(fd.OPERTYPE,"\0");

       data.item_id=PF_PARTMAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.PARTMAT,RecordIndex(3,data.value.integer)); strcat(fd.PARTMAT,"\0");

       data.item_id=PF_TOOLMAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.TOOLMAT,RecordIndex(2,data.value.integer)); strcat(fd.TOOLMAT,"\0");

       data.item_id=PF_DPT_CUT_MM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.DPT_CUT_MM=data.value.real;

       data.item_id=PF_INDEX1 ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.INDEX1=data.value.real;

       data.item_id=PF_INDEX2 ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.INDEX2=data.value.real;

       data.item_id=PF_SURF_SPEED_MPM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.SURF_SPEED_MPM=data.value.real;

       data.item_id=PF_FEED_MMPT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.FEED_MMPT=data.value.real;

       myFEEDS.SetAt(index, fd);

       data.item_id=PF_LIST;
       data.item_attr=UF_STYLER_LIST_DELETE;
       data.subitem_index=UF_STYLER_NO_SUB_INDEX;
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_attr=UF_STYLER_LIST_INSERT;
       data.indicator=UF_STYLER_STRING_VALUE ;
       for (i=0;i<myFEEDS.GetSize();i++) {
              fd = myFEEDS.GetAt(i);
              str[0]='\0'; fd.FormatList(str);
              data.subitem_index=i;
              data.value.string=str;
              irc=UF_STYLER_set_value(dialog_id,&data);
       }

       data.item_attr=UF_STYLER_VALUE;
       data.subitem_index=index;
       irc=UF_STYLER_set_value(dialog_id,&data);

     }


     if (change_mode==1) // edit or new
     {

       fd.Init();

       strcpy(fd.FORMAT,"DATA\0");

       data.item_attr=UF_STYLER_VALUE ;

       data.item_id=PF_LIBREF ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.LIBREF,data.value.string); strcat(fd.LIBREF,"\0");

       data.indicator=UF_STYLER_INTEGER_VALUE;
       data.count=1;

       data.item_id=PF_OPERTYPE ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.OPERTYPE,RecordIndex(1,data.value.integer)); strcat(fd.OPERTYPE,"\0");

       data.item_id=PF_PARTMAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.PARTMAT,RecordIndex(3,data.value.integer)); strcat(fd.PARTMAT,"\0");

       data.item_id=PF_TOOLMAT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       strcpy(fd.TOOLMAT,RecordIndex(2,data.value.integer)); strcat(fd.TOOLMAT,"\0");

       data.item_id=PF_DPT_CUT_MM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.DPT_CUT_MM=data.value.real;

       data.item_id=PF_INDEX1 ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.INDEX1=data.value.real;

       data.item_id=PF_INDEX2 ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.INDEX2=data.value.real;

       data.item_id=PF_SURF_SPEED_MPM ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.SURF_SPEED_MPM=data.value.real;

       data.item_id=PF_FEED_MMPT ;
       irc=UF_STYLER_ask_value(dialog_id,&data);
       fd.FEED_MMPT=data.value.real;

       myFEEDS.Add( fd );

       data.item_id=PF_LIST;
       data.item_attr=UF_STYLER_LIST_DELETE;
       data.subitem_index=UF_STYLER_NO_SUB_INDEX;
       irc=UF_STYLER_set_value(dialog_id,&data);

       data.item_attr=UF_STYLER_LIST_INSERT;
       data.indicator=UF_STYLER_STRING_VALUE ;
       for (i=0;i<myFEEDS.GetSize();i++) {
              fd = myFEEDS.GetAt(i);
              str[0]='\0'; fd.FormatList(str);
              data.subitem_index=i;
              data.value.string=str;
              irc=UF_STYLER_set_value(dialog_id,&data);
       }

       data.item_attr=UF_STYLER_VALUE;
       data.subitem_index=myFEEDS.GetSize()-1;;
       irc=UF_STYLER_set_value(dialog_id,&data);

     }

   break ;
   case 1: // Cancel

      PF_list_cb (dialog_id, client_data,callback_data) ;

   break ;
   default : uc1601(" ",TRUE);
  }

     UF_STYLER_free_value (&data) ;

     change_mode=-1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PF_LIST;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_BLO_ACT;
     data.value.integer=TRUE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_SW_EDIT;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_list_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_list_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int k ;
    FEEDS fd;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PF_LIST;
     data.item_attr=UF_STYLER_VALUE;
     data.indicator=UF_STYLER_INTEGER_VALUE ;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     k=data.value.integer ;

     fd = myFEEDS.GetAt(k);
     data.item_attr=UF_STYLER_VALUE ;

     data.item_id=PF_LIBREF ;
     data.value.string=fd.LIBREF;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_OPERTYPE ;
     data.subitem_index=FindIndex(1,k);
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_PARTMAT ;
     data.subitem_index=FindIndex(3,k);
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_TOOLMAT ;
     data.subitem_index=FindIndex(2,k);
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_DPT_CUT_MM ;
     data.value.real=fd.DPT_CUT_MM;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_INDEX1 ;
     data.value.real=fd.INDEX1;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_INDEX2 ;
     data.value.real=fd.INDEX2;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_SURF_SPEED_MPM ;
     data.value.real=fd.SURF_SPEED_MPM;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_FEED_MMPT ;
     data.value.real=fd.FEED_MMPT;
     irc=UF_STYLER_set_value(dialog_id,&data);

     UF_STYLER_free_value (&data) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PF_blo_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PF_blo_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i , k , l;
    char str[255] ;
    FEEDS fd;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PF_BLO_ACT;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     i=data.value.integer;

  data.item_id=PF_LIST;

  switch (i) {
   case 0 : // 

     change_mode=0; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PF_LIST;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_BLO_ACT;
     data.value.integer=FALSE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_SW_EDIT;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

   break;
   case 1 : //  

     change_mode=1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PF_LIST;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_BLO_ACT;
     data.value.integer=FALSE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PF_SW_EDIT;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_attr=UF_STYLER_VALUE;
     data.item_id=PF_LIBREF ;
     data.value.string=NewInd();
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_attr=UF_STYLER_FOCUS ;
     data.item_id=PF_DPT_CUT_MM ;
     irc=UF_STYLER_set_value(dialog_id,&data);

   break;
   case 2 : //  

         data.item_attr=UF_STYLER_VALUE;
         data.indicator=UF_STYLER_INTEGER_VALUE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         k=data.value.integer;

         i=Deletes(k);
         if (i<0) break ;

         l= myFEEDS.GetSize();
         if (l<0) break ;

         data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         for (i=0;i<l;i++) {
           fd = myFEEDS.GetAt(i);
           str[0]='\0'; fd.FormatList(str);
           data.item_attr=UF_STYLER_LIST_INSERT;
           data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }

         if (k<=1) k=0; else k--;
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=k;
         irc=UF_STYLER_set_value(dialog_id,&data);

         PF_list_cb (dialog_id,client_data,callback_data);

   break;
   case 3 : // 
         Lists_Help();
   break;
   case 4 : //  

         data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         PF_construct_cb (dialog_id,client_data,callback_data) ;

   break;
   case 5 : //  
         Saves();
   break;

   case 6 : break;
   default : uc1601(" ",TRUE);
  }

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}




// *.cpp: implementation of the * class.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

/*
**********************************************************
**********************************************************
*/
int Inits()
{
/*    UGS -  */
// {UGII_CAM_LIBRARY_TOOL_METRIC_DIR}tool_database.dat
// {UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR}\
//    cut_methods.dat
//    feeds_speeds.dat
//    part_materials.dat
//    tool_materials.dat

  char *ptr1;
  int fl ;
  FILE *rf;

  part_materials[0]='\0';
  tool_materials[0]='\0';
  feeds_speeds[0]='\0';
  cut_methods[0]='\0';
  SaveFlag=0;

  UF_translate_variable("UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR", &ptr1);
  if (ptr1==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR -  ", 1 );
     return(-1);
   }

/*          */
  feeds_speeds[0]='\0';
  strcpy(feeds_speeds,ptr1); strcat(feeds_speeds,"feeds_speeds.dat");

  fl=0;
  rf=fopen(feeds_speeds,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  rf=fopen(feeds_speeds,"a");
  if (rf==NULL) { SaveFlag=-1; } else { fclose(rf); }

  if (fl!=0) {
   return(-2);
  }

  cut_methods[0]='\0';
  strcpy(cut_methods,ptr1); strcat(cut_methods,"cut_methods.dat");
  rf=fopen(cut_methods,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  tool_materials[0]='\0';
  strcpy(tool_materials,ptr1); strcat(tool_materials,"tool_materials.dat");
  rf=fopen(tool_materials,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  part_materials[0]='\0';
  strcpy(part_materials,ptr1); strcat(part_materials,"part_materials.dat");
  rf=fopen(part_materials,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }


 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Loads()
{
   int i ;
   i=FeedsMaterial_Load();
   if (i<0) return (i);

   i=ToolMaterial_Load();
   if (i<0) ErrLoads( 2 );
   i=CutsMaterial_Load();
   if (i<0) ErrLoads( 1 );
   i=StockMaterial_Load();
   if (i<0) ErrLoads( 3 );

 return(0);
}

/*
**********************************************************
**********************************************************
*/
// 1: //CUTS   2: //TOOLMAT   3: //PARTMAT
int ErrLoads( int type)
{
  CUTS cts; //OPERTYPE
  TOOL_MATERIALS tm; //TOOLMAT
  PARTS prts; //PARTMAT

  switch (type) {
   case 1: //CUTS
      cts.Init(); myCUTS.Add( cts );
   break;
   case 2: //TOOLMAT
      tm.Init(); myTM.Add( tm );
   break;
   case 3: //PARTMAT
      prts.Init(); myPARTS.Add( prts );
   break;
   default:break;
  }

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int FeedsMaterial_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr], temp[mStr];
 FILE *rf;
 FEEDS tm;

 rf=fopen(feeds_speeds,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   feeds_speeds\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
   //i=0;
   //while (i<myTM.GetSize()) { delete myTM.GetAt(i++) ; }
   myFEEDS.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.OPERTYPE,stmp,k); //strcat(tm.OPERTYPE,"\0");
	   tm.OPERTYPE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.PARTMAT,stmp,k); //strcat(tm.PARTMAT,"\0");
	   tm.PARTMAT[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.TOOLMAT,stmp,k); //strcat(tm.TOOLMAT,"\0");
	   tm.TOOLMAT[k]='\0';

	   //DPT_CUT_IN
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //DPT_CUT_MM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.DPT_CUT_MM=atof(temp);

	   //INDEX1
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.INDEX1=atof(temp);

	   //INDEX2
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.INDEX2=atof(temp);

	   //SURF_SPEED_FPM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //SURF_SPEED_MPM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.SURF_SPEED_MPM=atof(temp);

	   //FEED_IPT
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //FEED_MMPT
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(temp,stmp,k);
	   } else {
	     strcpy(temp,stmp);
	   }
           //strcat(temp,"\0");
           temp[k]='\0';
           tm.FEED_MMPT=atof(temp);

           myFEEDS.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int ToolMaterial_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 TOOL_MATERIALS tm;

 rf=fopen(tool_materials,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   tool_materials\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
    myTM.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATNAM,stmp,k); //strcat(tm.MATNAM,"\0");
	   tm.MATNAM[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.MATDESC,stmp,k);
	   } else {
	     strcpy(tm.MATDESC,stmp);
	   }
           //strcat(tm.MATDESC,"\0");
           tm.MATDESC[k]='\0';

           myTM.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int StockMaterial_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 PARTS tm;

 rf=fopen(part_materials,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   part_materials\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
   //i=0;
   //while (i<myTM.GetSize()) { delete myTM.GetAt(i++) ; }
   myPARTS.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATCODE,stmp,k); //strcat(tm.MATCODE,"\0");
	   tm.MATCODE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATNAME,stmp,k); //strcat(tm.MATNAME,"\0");
	   tm.MATNAME[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.PARTMAT,stmp,k); //strcat(tm.PARTMAT,"\0");
	   tm.PARTMAT[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.HARDNESS,stmp,k); //strcat(tm.HARDNESS,"\0");
	   tm.HARDNESS[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.LIBREF,stmp,k);
	   } else {
	     strcpy(tm.LIBREF,stmp);
	   }
           //strcat(tm.LIBREF,"\0");
           tm.LIBREF[k]='\0';

           myPARTS.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int CutsMaterial_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 CUTS tm;

 rf=fopen(cut_methods,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   cut_methods\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
    myCUTS.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MODE,stmp,k); //strcat(tm.MODE,"\0");
	   tm.MODE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.NAME,stmp,k); //strcat(tm.NAME,"\0");
	   tm.NAME[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.DESCRIPTION,stmp,k);
	   } else {
	     strcpy(tm.DESCRIPTION,stmp);
	   }
           //strcat(tm.DESCRIPTION,"\0");
           tm.DESCRIPTION[k]='\0';

           myCUTS.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Saves()
{
 int index , l;
 char str[mStr] ;
 FILE *rf;
 FEEDS fd;

 if (SaveFlag==-1) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 l=myFEEDS.GetSize();
 if (l<=0) {
   uc1601("   \n  ...",TRUE);
   return(-1);
  }

 index=MessageBox(GetFocus(),(LPCTSTR)" ,      ?\n       -   \n   ..."," ...",MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
 if (index==IDNO) return (0) ;

 rf=fopen(feeds_speeds,"w");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-2);
 }

 fputs("# FEEDS_SPEEDS.DAT\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("#\n",rf);
 fputs("#   PURPOSE:\n",rf);
 fputs("#\n",rf);
 fputs("#      This is the database file used for defining Speeds and Feeds in\n",rf);
 fputs("#      an operation.\n",rf);
 fputs("#\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("##\n",rf);
 fputs("##  The data is organized in the format separated by |\n",rf);
 fputs("##  The fields in the database are in the following order:\n",rf);
 fputs("##\n",rf);
 fputs("##  LIBRF           - Unique record identifier\n",rf);
 fputs("##                     (Library Reference)\n",rf);
 fputs("##  OPERTYPE        - cutmthd_libref              Cut Method Library Reference\n",rf);
 fputs("##  PARTMAT         - part_material_libref Part   Part Material Library Reference\n",rf);
 fputs("##  TOOLMAT         - tool_material_libref        Tool Material Library Reference\n",rf);
 fputs("##  DPT_CUT_IN      - dpth_of_cut                 Depth_of_cut(inch)\n",rf);
 fputs("##  DPT_CUT_MM      - dpth_of_cut                 Depth_of_cut(mm)\n",rf);
 fputs("##  SURF_SPEED_FPM  - surface_speed               Suface Speed(FPM)\n",rf);
 fputs("##  SURF_SPEED_MPM  - surface_speed               Suface Speed(MPM)\n",rf);
 fputs("##  FEED_IPT        - feed_per_tooth              Feed per Tooth(IPT)\n",rf);
 fputs("##  FEED_MMPT       - feed_per_tooth              Feed per Tooth(MMPT)\n",rf);
 fputs("#\n",rf);
 fputs("#########################################################################\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);
 fputs("\nFORMAT LIBRF OPERTYPE PARTMAT TOOLMAT DPT_CUT_IN DPT_CUT_MM INDEX1 INDEX2 SURF_SPEED_FPM SURF_SPEED_MPM FEED_IPT FEED_MMPT\n\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);

 l=myFEEDS.GetSize();
 for (index=0;index<l;index++)
 {
   fd = myFEEDS.GetAt(index);
   str[0]='\0'; fd.FormatFile(str);
   fputs(str,rf);
 }

 fclose(rf);

 str[0]='\0'; sprintf(str," ...\n   %ld - .", l );
 MessageBox(GetFocus(),(LPCTSTR)str," ...",MB_ICONINFORMATION|MB_OK);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Lists_Help()
{
  int index ;
  char mes[mStr] ;
  char *release;
  FEEDS  fd;
  TOOL_MATERIALS tm;
  CUTS cts;
  PARTS prts;

  UF_UI_open_listing_window();

  UF_get_release (&release );
  mes[0]='\0'; sprintf(mes,"\n  Unigraphics - %s ",release); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n    ."); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n\t . (Win32 API, MFC, VC++6.0) "); UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n      :    ...");
  UF_UI_write_listing_window("\n#============================================================");

  UF_UI_write_listing_window("\n    :");
  UF_UI_write_listing_window("\n  '' -   ");
  UF_UI_write_listing_window("\n  ' ' -   ,    ");
  UF_UI_write_listing_window("\n  ' ' -   ");
  UF_UI_write_listing_window("\n  '/' -       .");
  UF_UI_write_listing_window("\n  ' ' -     ");
  UF_UI_write_listing_window("\n  ' ' -     ");

  UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

UF_UI_write_listing_window("\n\n           ");
UF_UI_write_listing_window("\n  ,       .");
UF_UI_write_listing_window("\n         .");
UF_UI_write_listing_window("\n         :");
UF_UI_write_listing_window("\n          ,   Feed Rates []");
UF_UI_write_listing_window("\n         Machining Parameters [ ].");
UF_UI_write_listing_window("\n        In the Feeds and Speeds [],");
UF_UI_write_listing_window("\n         Reset all from Table [   ].");
UF_UI_write_listing_window("\n               ASCII.");
UF_UI_write_listing_window("\n   ! ");
UF_UI_write_listing_window("\n     Reset all from Table [   ]     ,");
UF_UI_write_listing_window("\n         .");

UF_UI_write_listing_window("\n\n  :");
UF_UI_write_listing_window("\n speed in english = speed in metric * 1000 / (12 * 25.4).");
UF_UI_write_listing_window("\n feed_per_tooth in english = feed_per_tooth in metric/25.4 .");
UF_UI_write_listing_window("\n cut_depth () = cut_depth ()/25.4 .");
UF_UI_write_listing_window("\n   (/) = PI*D(,)*n(/)/1000. .");
UF_UI_write_listing_window("\n  (/) =  (/)*z( )*n(/).");
UF_UI_write_listing_window("\n   (%)   :");
UF_UI_write_listing_window("\n\t dbc_engage_percent     90");
UF_UI_write_listing_window("\n\t dbc_first_cut_percent  60");
UF_UI_write_listing_window("\n\t dbc_approach_percent    0");
UF_UI_write_listing_window("\n\t dbc_stepover_percent    0");
UF_UI_write_listing_window("\n\t dbc_retract_percent     0");
UF_UI_write_listing_window("\n\t dbc_departure_percent   0");
UF_UI_write_listing_window("\n\t dbc_return_percent      0");

  UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n       .");
  UF_UI_write_listing_window("\n      :\n");

  UF_UI_write_listing_window("\t  LIBRF           -    (Library Reference)\n");
  UF_UI_write_listing_window("\t  OPERTYPE        - cutmthd_libref              Cut Method Library Reference\n");
  UF_UI_write_listing_window("\t  PARTMAT         - part_material_libref Part   Part Material Library Reference\n");
  UF_UI_write_listing_window("\t  TOOLMAT         - tool_material_libref        Tool Material Library Reference \n");
  UF_UI_write_listing_window("\t  DPT_CUT_IN      - dpth_of_cut                   ()\n");
  UF_UI_write_listing_window("\t  DPT_CUT_MM      - dpth_of_cut                   (mm)\n");
  UF_UI_write_listing_window("\t  SURF_SPEED_FPM  - surface_speed                 (/)\n");
  UF_UI_write_listing_window("\t  SURF_SPEED_MPM  - surface_speed                 (/)\n");
  UF_UI_write_listing_window("\t  FEED_IPT        - feed_per_tooth                 (/)\n");
  UF_UI_write_listing_window("\t  FEED_MMPT       - feed_per_tooth                 (/)\n");
  UF_UI_write_listing_window("\n ");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myFEEDS.GetSize(), SaveFlag , feeds_speeds);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT LIBRF OPERTYPE PARTMAT TOOLMAT DPT_CUT_IN DPT_CUT_MM INDEX1 INDEX2 SURF_SPEED_FPM SURF_SPEED_MPM FEED_IPT FEED_MMPT");
  UF_UI_write_listing_window("\n#-----------------------------------------------------------\n");
  for (index=0;index<myFEEDS.GetSize();index++)
  {
    fd = myFEEDS.GetAt(index);
    mes[0]='\0';  fd.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n         'Tool Material'.");
  UF_UI_write_listing_window("\n      :");
  UF_UI_write_listing_window("\n\t  LIBREF   material_code         -   ");
  UF_UI_write_listing_window("\n\t  MATNAM   material_name         -    (    )");
  UF_UI_write_listing_window("\n\t  MATDESC  material_description  -  ");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myTM.GetSize(), SaveFlag , tool_materials);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT LIBRF MATNAM MATDESC\n");
  for (index=0;index<myTM.GetSize();index++)
  {
    tm = myTM.GetAt(index);
    mes[0]='\0';  tm.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n       .");
  UF_UI_write_listing_window("\n      :\n");
  UF_UI_write_listing_window("\t  MATCODE   material_code          -  \n");
  UF_UI_write_listing_window("\t  MATNAME   material_name          -   (   )\n");
  UF_UI_write_listing_window("\t  PARTMAT   material_description   -  \n");
  UF_UI_write_listing_window("\t  HARDNESS  material_hardness      -  \n");
  UF_UI_write_listing_window("\t  LIBREF    partmaterial_libref    -   \n");
  UF_UI_write_listing_window("\t                                     (Library Reference)\n");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myPARTS.GetSize(), SaveFlag , part_materials);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT MATCODE MATNAME PARTMAT HARDNESS LIBRF");
  UF_UI_write_listing_window("\n#-----------------------------------------------------------\n");
  for (index=0;index<myPARTS.GetSize();index++)
  {
    prts = myPARTS.GetAt(index);
    mes[0]='\0';  prts.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n        'Cut Methods'.");
  UF_UI_write_listing_window("\n      :");
  UF_UI_write_listing_window("\n\t  LIBREF   -    (Library Reference)");
  UF_UI_write_listing_window("\n\t  MODE     -   { MILL , DRILL , LATHE }");
  UF_UI_write_listing_window("\n\t  NAME     -   ");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myCUTS.GetSize(), SaveFlag , cut_methods);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT LIBRF MODE NAME DESCRIPTION");
  UF_UI_write_listing_window("\n#-----------------------------------------------------------\n");
  for (index=0;index<myCUTS.GetSize();index++)
  {
    cts = myCUTS.GetAt(index);
    mes[0]='\0';  cts.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n ");

  UF_UI_write_listing_window("\n#============================================================\n$$");

  //UF_UI_close_listing_window () ;

 return(0);

}


/*
**********************************************************
**********************************************************
*/
int Deletes(int index)
{
 if (index<0 || index>=myFEEDS.GetSize()) return (-1);

 try
 {
   myFEEDS.RemoveAt(index);
 }
  catch( char * str )
  {
    cerr << "Exception raised: " << str << '\n';
  }

 return (0);
}


/*
**********************************************************
**********************************************************
*/
char * NewInd()
{
   int index ;
   long int i;
   char ls[25], *ptr ;
   FEEDS  fd;

   /* Seed the random-number generator with current time so that
    * the numbers will be different every time we run. */
  srand( (unsigned)time( NULL ) );
  ls[0]='\0'; sprintf(ls,"FSDO_%04d",rand() );

  for (i=0,index=0;index<myFEEDS.GetSize();index++)
  {
    fd = myFEEDS.GetAt(index);
    if (0==strcmp( ls, fd.LIBREF)) {
    	if (i>1000) break;
    	ls[0]='\0'; sprintf(ls,"FSDO_%04d",rand() );
    	i++;
    	continue;
     }
  }

  ptr = (char *) malloc(strlen(ls)+5);
  strcpy(ptr,ls);strcat(ptr,"\0");
 return(ptr);
}


/*
**********************************************************
**********************************************************
*/
int Edit_New(int index)
{
    return (0);
}


/*
**********************************************************
**********************************************************
*/
int FindIndex( int type,long int index)
{
  long int i, step, len;
  char ls[26];
  FEEDS  fd;
  CUTS cts; //OPERTYPE
  TOOL_MATERIALS tm; //TOOLMAT
  PARTS prts; //PARTMAT

  fd = myFEEDS.GetAt(index);

  ls[0]='\0';
  len=-1;
  switch (type) {
   case 1: //CUTS
   len=myCUTS.GetSize();   sprintf(ls,"%.25s",strupr(fd.OPERTYPE) );
   break;
   case 2: //TOOLMAT
   len=myTM.GetSize();   sprintf(ls,"%.25s",strupr(fd.TOOLMAT) );
   break;
   case 3: //PARTMAT
   len=myPARTS.GetSize();   sprintf(ls,"%.25s",strupr(fd.PARTMAT) );
   break;
   default:break;
  }

  for (step=-1,i=0;i<len;i++)
  {

    if (type==1) { //CUTS
    	cts=myCUTS.GetAt(i);
        if (NULL!=strstr( ls, strupr(cts.LIBREF))) { step=i;break; }
    }
    if (type==2) { //TOOLMAT
    	tm=myTM.GetAt(i);
        if (NULL!=strstr( ls, strupr(tm.LIBREF))) { step=i;break; }
    }
    if (type==3) { //PARTMAT
    	prts=myPARTS.GetAt(i);
        if (NULL!=strstr( ls, strupr(prts.LIBREF))) { step=i;break; }
    }

  }

  if (step<0) step=0;
  return(step);
}

/*
**********************************************************
**********************************************************
*/
char * RecordIndex( int type,long int i)
{
  long int len;
  char ls[26], *ptr ;
  CUTS cts; //OPERTYPE
  TOOL_MATERIALS tm; //TOOLMAT
  PARTS prts; //PARTMAT

  ls[0]='\0';
  switch (type) {
   case 1: //CUTS
    len=myCUTS.GetSize();
    if (i<0 || i>=len) { sprintf(ls,"NULL"); break; }
    cts=myCUTS.GetAt(i);
    sprintf(ls,"%.20s",strupr(cts.LIBREF) );
   break;
   case 2: //TOOLMAT
    len=myTM.GetSize();
    if (i<0 || i>=len) { sprintf(ls,"NULL"); break; }
    tm=myTM.GetAt(i);
    sprintf(ls,"%.20s",strupr(tm.LIBREF) );
   break;
   case 3: //PARTMAT
    len=myPARTS.GetSize();
    if (i<0 || i>=len) { sprintf(ls,"NULL"); break; }
    prts=myPARTS.GetAt(i);
     sprintf(ls,"%.20s",strupr(prts.LIBREF) );
   break;
   default:break;
  }

  ptr = (char *) malloc(strlen(ls)+1);
  strcpy(ptr,ls);strcat(ptr,"\0");
  return(ptr);
}

