//////////////////////////////////////////////////////////////////////////////
//
//  libstockmat.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

//  Include files
//  Include files
#include <stdio.h>
#include <uf_defs.h>
#include <uf_styler.h>
#include <uf_mb.h>
#include <uf.h>
#include <uf_exit.h>
#include <uf_ui.h>

#include "afxtempl.h"

#include <windows.h>

#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
#   include <iostream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
	using std::cerr;
#else
#	include <strstream.h>
#   include <iostream.h>
#endif

/* These include files are needed for the following template code.            */
#include "libstockmat.h"

#define mStr  255

class MATERIALS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; // partmaterial_libref    - Unique record identifier
	char MATCODE[50]; //MATCODE   material_code          - Material Code
	char MATNAME[50]; //  MATNAME   material_name          - Material Name (appears on the label)
	char PARTMAT[133]; // PARTMAT   material_description   - Material Description
	char HARDNESS[20]; // HARDNESS  material_hardness      - Material Hardness

 MATERIALS ()
 {
   Init();
 }

 virtual ~MATERIALS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MATCODE[0]='\0'; MATNAME[0]='\0'; PARTMAT[0]='\0'; HARDNESS[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

// |||:
 void FormatList(char * str)
 { sprintf(str,"%.50s  =  %.50s  =  %.20s  =  %.133s",MATCODE , MATNAME, HARDNESS , PARTMAT );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.50s|%.50s|%.133s|%.20s|%.20s\n", FORMAT,MATCODE,MATNAME,PARTMAT,HARDNESS,LIBREF );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.50s|%.50s|%.133s|%.20s|%.20s\n", FORMAT,MATCODE,MATNAME,PARTMAT,HARDNESS,LIBREF );  }

} ;

 CArray < MATERIALS, MATERIALS& > myTM ;

 char part_materials[255];
 int SaveFlag;

 int change_mode; // edit or new


/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t PM_cbs */
#define PM_CB_COUNT ( 11 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with PM_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t PM_cbs[PM_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, PM_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, PM_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, PM_cancel_cb},
 {PM_LIBREF             , UF_STYLER_ACTIVATE_CB     , 0, PM_partmaterial_libref_cb},
 {PM_MATCODE            , UF_STYLER_ACTIVATE_CB     , 0, PM_material_code_cb},
 {PM_MATNAME            , UF_STYLER_ACTIVATE_CB     , 0, PM_material_name_cb},
 {PM_HARDNESS           , UF_STYLER_ACTIVATE_CB     , 0, PM_material_hardness_cb},
 {PM_PARTMAT            , UF_STYLER_ACTIVATE_CB     , 0, PM_material_description_cb},
 {PM_BLO_ACTION         , UF_STYLER_ACTIVATE_CB     , 0, PM_edit_act_cb},
 {PM_LIST               , UF_STYLER_ACTIVATE_CB     , 0, PM_list_cb},
 {PM_BLO_ACT            , UF_STYLER_ACTIVATE_CB     , 0, PM_blo_act_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};


//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int i ;
    int response=0;
    char fail_message[133];
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
     i=Material_Init();
     if (i<0) {
        fail_message[0]='\0';
        sprintf(fail_message,"       \n    .");
        MessageBeep(0);
        MessageBox(GetFocus(),(LPCTSTR)fail_message," - ->.",MB_ICONSTOP|MB_OK);
     }

    if ( ( errorCode = UF_STYLER_create_dialog ( "libstockmat.dlg",
           PM_cbs,      /* Callbacks from dialog */
           PM_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          /* Get the user function fail message based on the fail code.*/
          fail_message[0]='\0';
          UF_get_fail_message(errorCode, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );
          uc1601("   'libstockmat.dlg'  ...",TRUE);
    }


        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode=0;
}

//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern int ufusr_ask_unload (void)
{
     /* unload immediately after application exits*/
     return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     //return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}



/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        // Construct a buffer to hold the text.
        ostrstream error_message;

        // Initialize the buffer with the required text.
        error_message << endl
                      << "Error:" << endl
                      << message
                      << endl << endl << ends;

	    // Write the message to standard error
        cerr << error_message.str();
    }
}



/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: PM_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    long int i , l;
    char str[255] ; // CString
    //char ls[255];
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     change_mode=-1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;
     data.item_id=PM_SW_EDIT;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     i=Material_Load();
     if (i==0) {
       l= myTM.GetSize();
       if (l>0) {
	 data.item_id=PM_LIST;
         data.item_attr=UF_STYLER_LIST_INSERT;
         data.subitem_index=UF_STYLER_NO_SUB_INDEX;
         data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
         for (i=0;i<l;i++) {
           tm = myTM.GetAt(i);
           str[0]='\0'; tm.FormatList(str);
           data.subitem_index=i;
	   data.value.string=str; //(char**)ls;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=0;
         irc=UF_STYLER_set_value(dialog_id,&data);

         tm = myTM.GetAt(0);
         data.item_attr=UF_STYLER_VALUE ;

         data.item_id=PM_LIBREF ;
         data.value.string=tm.LIBREF;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PM_MATCODE ;
         data.value.string=tm.MATCODE;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PM_MATNAME ;
         data.value.string=tm.MATNAME;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PM_HARDNESS ;
         data.value.string=tm.HARDNESS;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_id=PM_PARTMAT ;
         data.value.string=tm.PARTMAT;
         irc=UF_STYLER_set_value(dialog_id,&data);

       }
     }

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_partmaterial_libref_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_partmaterial_libref_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_material_code_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_material_code_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_material_name_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_material_name_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_material_hardness_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_material_hardness_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_material_description_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_material_description_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_edit_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_edit_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i , index;
    char str[mStr] ;
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PM_BLO_ACTION;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     i=data.value.integer;

  switch (i) {
   case 0 : // OK

     if (change_mode==0) // edit or new
     {

         data.item_id=PM_LIST;
         data.item_attr=UF_STYLER_VALUE;
         data.indicator=UF_STYLER_INTEGER_VALUE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         index=data.value.integer ;

         if (index<0) break ;

         tm = myTM.GetAt(index);

         data.item_attr=UF_STYLER_VALUE ;

         data.item_id=PM_LIBREF ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.LIBREF,data.value.string); strcat(tm.LIBREF,"\0");

         data.item_id=PM_MATCODE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.MATCODE,data.value.string); strcat(tm.MATCODE,"\0");

         data.item_id=PM_MATNAME ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.MATNAME,data.value.string); strcat(tm.MATNAME,"\0");

         data.item_id=PM_HARDNESS ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.HARDNESS,data.value.string); strcat(tm.HARDNESS,"\0");

         data.item_id=PM_PARTMAT ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.PARTMAT,data.value.string); strcat(tm.PARTMAT,"\0");

         myTM.SetAt(index, tm);

         //PM_list_cb (dialog_id, client_data,callback_data) ;

         data.item_id=PM_LIST;
         data.item_attr=UF_STYLER_LIST_DELETE;
         data.subitem_index=UF_STYLER_NO_SUB_INDEX;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_attr=UF_STYLER_LIST_INSERT;
         data.indicator=UF_STYLER_STRING_VALUE ;
         for (i=0;i<myTM.GetSize();i++) {
              tm = myTM.GetAt(i);
              str[0]='\0'; tm.FormatList(str);
              data.subitem_index=i;
              data.value.string=str;
              irc=UF_STYLER_set_value(dialog_id,&data);
         }

         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=index;
         irc=UF_STYLER_set_value(dialog_id,&data);

     }


     if (change_mode==1) // edit or new
     {
         tm.Init();

         strcpy(tm.FORMAT,"DATA\0");

         data.item_attr=UF_STYLER_VALUE ;

         data.item_id=PM_LIBREF ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.LIBREF,data.value.string); strcat(tm.LIBREF,"\0");

         data.item_id=PM_MATCODE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.MATCODE,data.value.string); strcat(tm.MATCODE,"\0");

         data.item_id=PM_MATNAME ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.MATNAME,data.value.string); strcat(tm.MATNAME,"\0");

         data.item_id=PM_HARDNESS ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.HARDNESS,data.value.string); strcat(tm.HARDNESS,"\0");

         data.item_id=PM_PARTMAT ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         strcpy(tm.PARTMAT,data.value.string); strcat(tm.PARTMAT,"\0");

         myTM.Add( tm );

         data.item_id=PM_LIST;
         data.item_attr=UF_STYLER_LIST_DELETE;
         data.subitem_index=UF_STYLER_NO_SUB_INDEX;
         irc=UF_STYLER_set_value(dialog_id,&data);

         data.item_attr=UF_STYLER_LIST_INSERT;
         data.indicator=UF_STYLER_STRING_VALUE ;
         for (i=0;i<myTM.GetSize();i++) {
           tm = myTM.GetAt(i);
           str[0]='\0'; tm.FormatList(str);
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }

         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=myTM.GetSize()-1;
         irc=UF_STYLER_set_value(dialog_id,&data);

     }

   break ;
   case 1: // Cancel

      PM_list_cb (dialog_id, client_data,callback_data) ;

   break ;
   default : uc1601(" ",TRUE);
  }


     UF_STYLER_free_value (&data) ;

     change_mode=-1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PM_LIST;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_BLO_ACT;
     data.value.integer=TRUE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_SW_EDIT;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);


     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_list_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_list_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int k ;
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PM_LIST;
     data.item_attr=UF_STYLER_VALUE;
     data.indicator=UF_STYLER_INTEGER_VALUE ;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     k=data.value.integer ;

     tm = myTM.GetAt(k);
     data.item_attr=UF_STYLER_VALUE ;

     data.item_id=PM_LIBREF ;
     data.value.string=tm.LIBREF;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_MATCODE ;
     data.value.string=tm.MATCODE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_MATNAME ;
     data.value.string=tm.MATNAME;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_HARDNESS ;
     data.value.string=tm.HARDNESS;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_PARTMAT ;
     data.value.string=tm.PARTMAT;
     irc=UF_STYLER_set_value(dialog_id,&data);

     UF_STYLER_free_value (&data) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PM_blo_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PM_blo_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
    UF_STYLER_item_value_type_t data  ;
    int irc ;
    int i , k , l;
    char str[255] ;
    MATERIALS tm ;

     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     data.item_id=PM_BLO_ACT;
     data.item_attr=UF_STYLER_VALUE;
     irc=UF_STYLER_ask_value(dialog_id,&data);
     i=data.value.integer;

  data.item_id=PM_LIST;

  switch (i) {
   case 0 : // 

     change_mode=0; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PM_LIST;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_BLO_ACT;
     data.value.integer=FALSE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_SW_EDIT;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

   break;
   case 1 : //  

     change_mode=1; // edit or new

     data.item_attr=UF_STYLER_SENSITIVITY ;

     data.item_id=PM_LIST;
     data.value.integer=FALSE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_BLO_ACT;
     data.value.integer=FALSE;
     data.subitem_index=UF_STYLER_NO_SUB_INDEX;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_id=PM_SW_EDIT;
     data.value.integer=TRUE;
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_attr=UF_STYLER_VALUE;
     data.item_id=PM_LIBREF ;
     data.value.string=Material_NewInd();
     irc=UF_STYLER_set_value(dialog_id,&data);

     data.item_attr=UF_STYLER_FOCUS ;
     data.item_id=PM_MATCODE ;
     irc=UF_STYLER_set_value(dialog_id,&data);

   break;
   case 2 : //  

         data.item_attr=UF_STYLER_VALUE;
         data.indicator=UF_STYLER_INTEGER_VALUE ;
         irc=UF_STYLER_ask_value(dialog_id,&data);
         k=data.value.integer;

         i=Material_Delete(k);
         if (i<0) break ;

         l= myTM.GetSize();
         if (l<0) break ;

         data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         for (i=0;i<l;i++) {
           tm = myTM.GetAt(i);
           str[0]='\0'; tm.FormatList(str);
           data.item_attr=UF_STYLER_LIST_INSERT;
           data.indicator=UF_STYLER_STRING_VALUE ; // UF_STYLER_STRING_PTR_VALUE ;
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }

         if (k<=1) k=0; else k--;
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=k;
         irc=UF_STYLER_set_value(dialog_id,&data);
   break;
   case 3 : // 
         Material_List_Help();
   break;
   case 4 : //  

	 data.item_attr=UF_STYLER_LIST_DELETE;
	 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	 irc=UF_STYLER_set_value(dialog_id,&data);

         i=Material_Load();
         if (i<0) break ;

         l= myTM.GetSize();
         if (l<=0) break ;

         data.item_attr=UF_STYLER_LIST_INSERT;
         data.indicator=UF_STYLER_STRING_VALUE ;
         for (i=0;i<l;i++) {
           tm = myTM.ElementAt(i);
           str[0]='\0';tm.FormatList(str);
           data.subitem_index=i;
           data.value.string=str;
           irc=UF_STYLER_set_value(dialog_id,&data);
         }
         data.item_attr=UF_STYLER_VALUE;
         data.subitem_index=0;
         irc=UF_STYLER_set_value(dialog_id,&data);
   break;
   case 5 : //  
         Material_Save();
   break;

   case 6 : break;
   default : uc1601(" ",TRUE);
  }

     UF_STYLER_free_value (&data) ;
     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}




// Material.cpp: implementation of the Material class.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

/*
**********************************************************
**********************************************************
*/
int Material_Init()
{
/*    UGS -  */
// {UGII_CAM_LIBRARY_TOOL_METRIC_DIR}tool_database.dat
// {UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR}\
//    cut_methods.dat
//    feeds_speeds.dat
//    part_materials.dat
//    tool_materials.dat

  char *ptr1;
  int fl ;
  FILE *rf;

  part_materials[0]='\0';
  SaveFlag=0;

  UF_translate_variable("UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR", &ptr1);
  if (ptr1==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR -  ", 1 );
     return(-1);
   }

/*          */
  part_materials[0]='\0';
  strcpy(part_materials,ptr1); strcat(part_materials,"part_materials.dat");

  fl=0;
  rf=fopen(part_materials,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  rf=fopen(part_materials,"a");
  if (rf==NULL) { SaveFlag=-1; } else { fclose(rf); }

  if (fl!=0) {
   return(-2);
  }

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 MATERIALS tm;

 rf=fopen(part_materials,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 //myTM.SetSize(50);
 try
 {
   //i=0;
   //while (i<myTM.GetSize()) { delete myTM.GetAt(i++) ; }
   myTM.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATCODE,stmp,k); //strcat(tm.MATCODE,"\0");
	   tm.MATCODE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATNAME,stmp,k); //strcat(tm.MATNAME,"\0");
	   tm.MATNAME[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.PARTMAT,stmp,k); //strcat(tm.PARTMAT,"\0");
	   tm.PARTMAT[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.HARDNESS,stmp,k); //strcat(tm.HARDNESS,"\0");
	   tm.HARDNESS[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.LIBREF,stmp,k);
	   } else {
	     strcpy(tm.LIBREF,stmp);
	   }
           //strcat(tm.LIBREF,"\0");
           tm.LIBREF[k]='\0';

           myTM.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Delete(int index)
{
 if (index<0 || index>=myTM.GetSize()) return (-1);

 try
 {
   myTM.RemoveAt(index);
 }
  catch( char * str )
  {
    cerr << "Exception raised: " << str << '\n';
  }

 return (0);
}


/*
**********************************************************
**********************************************************
*/
int Material_Save()
{
 int index , l;
 char str[mStr] ;
 FILE *rf;
 MATERIALS tm;

 if (SaveFlag==-1) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 l=myTM.GetSize();
 if (l<=0) {
   uc1601("   \n  ...",TRUE);
   return(-1);
  }

 index=MessageBox(GetFocus(),(LPCTSTR)" ,      ?\n       -   \n   ..."," ...",MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
 if (index==IDNO) return (0) ;

 rf=fopen(part_materials,"w");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-2);
 }

 fputs("# PART_MATERIALS.DAT\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("#\n",rf);
 fputs("#   PURPOSE:\n",rf);
 fputs("#\n",rf);
 fputs("#      This is the database file used for defining Part Material.\n",rf);
 fputs("#\n",rf);
 fputs("##########################################################################\n",rf);
 fputs("##\n",rf);
 fputs("##  The following key words for Attribute ids are defined\n",rf);
 fputs("##\n",rf);
 fputs("##  MATCODE   material_code          - Material Code\n",rf);
 fputs("##  MATNAME   material_name          - Material Name (appears on the label)\n",rf);
 fputs("##  PARTMAT   material_description   - Material Description\n",rf);
 fputs("##  HARDNESS  material_hardness      - Material Hardness\n",rf);
 fputs("##  LIBREF    partmaterial_libref    - Unique record identifier\n",rf);
 fputs("##                                     (Library Reference)\n",rf);
 fputs("#########################################################################\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);
 fputs("\nFORMAT MATCODE MATNAME PARTMAT HARDNESS LIBRF\n\n",rf);
 fputs("#----------------+-------------------------------+-------------\n",rf);

 l=myTM.GetSize();
 for (index=0;index<l;index++)
 {
   tm = myTM.GetAt(index);
   str[0]='\0'; tm.FormatFile(str);
   fputs(str,rf);
 }

 fclose(rf);

 str[0]='\0'; sprintf(str," ...\n   %ld - .", l );
 MessageBox(GetFocus(),(LPCTSTR)str," ...",MB_ICONINFORMATION|MB_OK);

 return(0);
}



/*
**********************************************************
**********************************************************
*/
int Material_List_Help()
{
  int index ;
  char mes[mStr] ;
  char *release;
  MATERIALS tm;

  UF_UI_open_listing_window();

  UF_get_release (&release );
  mes[0]='\0'; sprintf(mes,"\n  Unigraphics - %s ",release); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n  ."); UF_UI_write_listing_window(mes);

  mes[0]='\0'; sprintf(mes,"\n\t . (Win32 API, MFC, VC++6.0) "); UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n      :     .");
  UF_UI_write_listing_window("\n#============================================================");

  UF_UI_write_listing_window("\n    :");
  UF_UI_write_listing_window("\n  '' -   ");
  UF_UI_write_listing_window("\n  ' ' -   ");
  UF_UI_write_listing_window("\n  ' ' -   ,    ");
  UF_UI_write_listing_window("\n  '/' -       .");
  UF_UI_write_listing_window("\n  ' ' -     ");
  UF_UI_write_listing_window("\n  ' ' -     ");

  UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

UF_UI_write_listing_window("\n\n\t  Accessing the Part Material Library [    ]");
UF_UI_write_listing_window("\n           Workpiece [].");
UF_UI_write_listing_window("\n        :");
UF_UI_write_listing_window("\n               .");
UF_UI_write_listing_window("\n       MB3      ,");
UF_UI_write_listing_window("\n                 ");
UF_UI_write_listing_window("\n        Insert [] -> Geometry [].");
UF_UI_write_listing_window("\n       Insert Geometry [ ]");
UF_UI_write_listing_window("\n      :");
UF_UI_write_listing_window("\n       1.   Material []    Search Result [ ]");
UF_UI_write_listing_window("\n             .");
UF_UI_write_listing_window("\n       2.      Part Material Library [  ].");

  UF_UI_write_listing_window("\n\n#-----------------------------------------------------------");

  UF_UI_write_listing_window("\n\n      .");
  UF_UI_write_listing_window("\n      :\n");
  UF_UI_write_listing_window("\t  MATCODE   material_code          -  \n");
  UF_UI_write_listing_window("\t  MATNAME   material_name          -   (   )\n");
  UF_UI_write_listing_window("\t  PARTMAT   material_description   -  \n");
  UF_UI_write_listing_window("\t  HARDNESS  material_hardness      -  \n");
  UF_UI_write_listing_window("\t  LIBREF    partmaterial_libref    -   \n");
  UF_UI_write_listing_window("\t                                     (Library Reference)\n");
  UF_UI_write_listing_window("\n ");

  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n#   :");
  mes[0]='\0';
  sprintf(mes,"\n# Count=%ld\n# SaveFlag=%d\n# Name=%s ",myTM.GetSize(), SaveFlag , part_materials);
  UF_UI_write_listing_window(mes);
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\nFORMAT MATCODE MATNAME PARTMAT HARDNESS LIBRF");
  UF_UI_write_listing_window("\n#-----------------------------------------------------------\n");
  for (index=0;index<myTM.GetSize();index++)
  {
    tm = myTM.GetAt(index);
    mes[0]='\0';  tm.FormatHelp(mes);
    UF_UI_write_listing_window(mes);
  }
  UF_UI_write_listing_window("\n#-----------------------------------------------------------");
  UF_UI_write_listing_window("\n ");
  UF_UI_write_listing_window("\n#============================================================\n$$");

  //UF_UI_close_listing_window () ;

 return(0);

}


/*
**********************************************************
**********************************************************
*/
char * Material_NewInd()
{
   int index ;
   long int i;
   char ls[25], *ptr ;
   MATERIALS tm;

   /* Seed the random-number generator with current time so that
    * the numbers will be different every time we run. */
  srand( (unsigned)time( NULL ) );
  ls[0]='\0'; sprintf(ls,"MAT0_%04d",rand() );

  for (i=0,index=0;index<myTM.GetSize();index++)
  {
    tm = myTM.GetAt(index);
    if (0==strcmp( ls, tm.LIBREF)) {
    	if (i>1000) break;
    	ls[0]='\0'; sprintf(ls,"MAT0_%04d",rand() );
    	i++;
    	continue;
     }
  }

  ptr = (char *) malloc(strlen(ls)+5);
  strcpy(ptr,ls);strcat(ptr,"\0");
 return(ptr);
}


/*
**********************************************************
**********************************************************
*/
int Material_Edit_New(int index)
{
    return (0);
}



