//////////////////////////////////////////////////////////////////////////////
//
//  libtools.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

//  Include files
#include <stdio.h>
#include <malloc.h>

#include <uf_defs.h>
#include <uf.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>
#include <uf_mb.h>

#include "afxtempl.h"

#include <windows.h>

#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
#   include <iostream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
	using std::cerr;
#else
#	include <strstream.h>
#   include <iostream.h>
#endif

/* These include files are needed for the following template code.            */
#include "libtools.h"

#define mStr  255

class TOOLS
{
 public :
	//# The following attributes are common to all tool types and subtypes:
	char FORMAT[10];
	char LIBRF[25]; // - Unique identifier (so called library refernce) for a tool
	int  T; // - Classification type of a tool.
	int  ST; // - Classification subtype of a tool.
	int  UGT; // - UG Tool Type
	int  UGST; // - UG Tool SubType
	char DESCR[50]; // - Short tool description
	//char TOOLMAT[20]; //- tool_material_libref        Tool Material Library Reference
	char MATREF[25]; // - Reference to cutter material table (tool_materials.dat)
	char MATDES[50]; // - Cutter material description
	int TLNUM; // - Tool Number
	int ADJREG; // - Tool Length Adjust Register
	int CUTCOMREG; // - Tool CUTCOM Register
	char HLD[10]; /* - Holding system for mounting tool to machine
		       #   HLD is used to check whether the tool can be mounted
		       #   to a pocket of the tool carrier of the machine.
		       #   The HLD value here has to match one of the
		       #   Holding Systems specified in the pocket definition in
		       #   the postprocessor .def file of the machine.
		       */
        char HLDDES[50];  //  - Holding system description
        /***************************/
       // struct {
         double  DIA;
         double  FN;
         double  HEI;
         double  ZOFF;
         double  DROT;
         double  FLEN;
           double  D2 ; //1+ 2-
           double  SDIA ; //1- 2+
           double  LCOR ; //1- 2+
           double  UCOR ; //1- 2+
         double  TAPA; //1- 2-
           double  TIPA; //10+
         double  COR1; //2-
         double  CX1;
         double  CY1;
         double  COR2;
         double  CX2;
         double  CY2;
         double  HDIA;
         double  HLEN;
         double  HTAP;
         double  HOFF;
         int   THRDS;
         char  THRDES[50];
         double PIT;
       // } mill ;

 TOOLS ()
 {
   Init();
 }

 virtual ~TOOLS () { ; }

 void Init() {
   FORMAT[0]='\0'; LIBRF[0]='\0';
   T=0; ST=0;UGT=0;UGST=0;
   DESCR[0]='\0';
   /* TOOLMAT[0]='\0';*/ MATREF[0]='\0';MATDES[0]='\0';
   TLNUM=0;ADJREG=0;CUTCOMREG=0;
   HLD[0]='\0';HLDDES[0]='\0';
 }

 void FormatFile_mill1(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,TAPA,COR1,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill2(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,D2,COR1,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill3(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,SDIA,LCOR,UCOR,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill_threads(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf\
   		  %02d|%.50s\
   		  |%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,
                      THRDS,THRDES \
                      PIT,FLEN,TAPA,COR1,SDIA,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill10(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,TAPA,TIPA,COR1,CX1,CY1,COR2,CX2,CY2,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill7(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\
   		  %.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,TAPA,TIPA,COR1,CX1,CY1,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatFile_mill5(char * str)
 {
   sprintf(str,"%.5s|%.20s|%02d|%02d|%02d|%02d|%.50s|%.20s|%.50s|\
   		  %d|%d|%d|%.20s|%.50s|\
   		  %.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf|%.3lf\n", \
                      FORMAT,LIBRF, \
                      T,ST,UGT,UGST, \
                      DESCR, \
                      MATREF,MATDES, \
                      TLNUM, ADJREG, CUTCOMREG \
                      HLD,HLDDES , \
                      DIA,FN,HEI,ZOFF,DROT,FLEN,TIPA,TAPA,COR1,HDIA,HLEN,HTAP,HOFF );
 }

 void FormatList(char * str)
 {
  //   sprintf(str,"%.20s  =  %.3lf  i1=  %.2lf  i2=  %.2lf =  %.3lf  =  %.3lf ",LIBREF, DPT_CUT_MM, INDEX1 ,INDEX2, SURF_SPEED_MPM , FEED_MMPT );
 }

} ;

 CArray < TOOLS, TOOLS& > myTOOLS ;
 char tool_database[255];
 int SaveFlag;

/*========================================================================================*/

 int change_mode; // edit or new

/*========================================================================================*/

/*========================================================================================*/
class TOOL_MATERIALS
{
 public :
	char FORMAT[10];
	char LIBREF[20]; //material_code         - Unique record identifier
	char MATNAM[50]; //material_name         - Material Name  (appears on the label)
	char MATDESC[133]; //material_description  - Material Description

 TOOL_MATERIALS ()
 {
   Init();
 }

 virtual ~TOOL_MATERIALS () { ; }

 void Init() { FORMAT[0]='\0'; LIBREF[0]='\0'; MATNAM[0]='\0'; MATDESC[0]='\0'; }

 char * FormatChar(CString str)
 {
   char ls[255], *ptr;
   ls[0]='\0'; sprintf(ls,"%s",str ); //(LPCTSTR)
   ptr = (char *) malloc(strlen(ls)+5);
   strcpy(ptr,ls);strcat(ptr,"\0");
   return(ptr);
 }

 void FormatList(char * str)
 { sprintf(str,"%.50s  = %.80s",MATNAM, MATDESC );  }

 void FormatFile(char * str)
 { sprintf(str,"%.5s|%.20s|%.50s|%.133s\n", FORMAT , LIBREF , MATNAM , MATDESC );  }

 void FormatHelp(char * str)
 { sprintf(str,"%.5s|%.20s|%.50s|%.133s\n", FORMAT, LIBREF, MATNAM, MATDESC );  }

} ;

 CArray < TOOL_MATERIALS, TOOL_MATERIALS& > myTM ;
 char tool_materials[255];

/*========================================================================================*/

/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t PT_cbs */
#define PT_CB_COUNT ( 7 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with PT_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t PT_cbs[PT_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, PT_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, PT_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CANCEL_CB       , 0, PT_cancel_cb},
 {PT_OPTION_TYPE        , UF_STYLER_ACTIVATE_CB     , 0, PT_type_cb},
 {PT_TOOL_GRAPHICS      , UF_STYLER_ACTIVATE_CB     , 0, PT_tool_cb},
 {PT_LIST_TOOLS         , UF_STYLER_ACTIVATE_CB     , 0, PT_list_cb},
 {PT_BLO_ACT            , UF_STYLER_ACTIVATE_CB     , 1, PT_blo_act_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};



/*--------------------------------------------------------------------------
UF_MB_styler_actions_t contains 4 fields.  These are defined as follows:

Field 1 : the name of your dialog that you wish to display.
Field 2 : any client data you wish to pass to your callbacks.
Field 3 : your callback structure.
Field 4 : flag to inform menubar of your dialog location.  This flag MUST
          match the resource set in your dialog!  Do NOT ASSUME that changing
          this field will update the location of your dialog.  Please use the
          UIStyler to indicate the position of your dialog.
--------------------------------------------------------------------------*/
static UF_MB_styler_actions_t actions[] = {
    { "libtools.dlg",  NULL,   PT_cbs,  UF_MB_STYLER_IS_NOT_TOP },
    { NULL,  NULL,  NULL,  0 } /* This is a NULL terminated list */
};



//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int i ;
    int response=0;
    char fail_message[133];
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
     i=Inits();
     if (i<0) {
        fail_message[0]='\0';
        sprintf(fail_message," ''      \n    .");
        MessageBeep(0);
        MessageBox(GetFocus(),(LPCTSTR)fail_message," - ->.",MB_ICONSTOP|MB_OK);
        errorCode = UF_terminate();
        PrintErrorMessage( errorCode );
        *returnCode=0;
        return;
     }

    if ( ( errorCode = UF_STYLER_create_dialog ( "libtools.dlg",
           PT_cbs,      /* Callbacks from dialog */
           PT_CB_COUNT, /* number of callbacks*/
           NULL,        /* This is your client data */
           &response ) ) != 0 )
    {
          /* Get the user function fail message based on the fail code.*/
          fail_message[0]='\0';
          UF_get_fail_message(errorCode, fail_message);
          UF_UI_set_status (fail_message);
          printf ( "%s\n", fail_message );
          uc1601("   'libtools.dlg' \n    ?...",TRUE);
    }

        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode=0;
}

//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern int ufusr_ask_unload (void)
{
     /* unload immediately after application exits*/
     //return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     /*return ( UF_UNLOAD_UG_TERMINATE ); */
}


/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}


/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        // Construct a buffer to hold the text.
        ostrstream error_message;

        // Initialize the buffer with the required text.
        error_message << endl
                      << "Error:" << endl
                      << message
                      << endl << endl << ends;

	    // Write the message to standard error
        cerr << error_message.str();
    }
}


/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: PT_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_ok_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_ok_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a ok callback.*/
    /* return ( UF_UI_CB_EXIT_DIALOG );                    */
    return (UF_UI_CB_EXIT_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_cancel_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_cancel_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, terminate dialog             */
    /* It is STRONGLY recommended that you exit your       */
    /* callback with UF_UI_CB_EXIT_DIALOG in a cancel call */
    /* back rather than UF_UI_CB_CONTINUE_DIALOG.          */
    return ( UF_UI_CB_EXIT_DIALOG );

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_type_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_type_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_tool_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_tool_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_list_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_list_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: PT_blo_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the UG/Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int PT_blo_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}



// *.cpp: implementation of the * class.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

/*
**********************************************************
**********************************************************
*/
int Inits()
{
/*    UGS -  */
// {UGII_CAM_LIBRARY_TOOL_METRIC_DIR}tool_database.dat
// {UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR}\
//    cut_methods.dat
//    feeds_speeds.dat
//    part_materials.dat
//    tool_materials.dat

  char *ptr1,*ptr2;
  int fl ;
  FILE *rf;

  tool_database[0]='\0';
  tool_materials[0]='\0';

  SaveFlag=0;

/*     */
  UF_translate_variable("UGII_CAM_LIBRARY_TOOL_METRIC_DIR", &ptr1);
  if (ptr1==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_TOOL_METRIC_DIR -  ", 1 );
     return(-1);
   }

  UF_translate_variable("UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR", &ptr2);
  if (ptr2==NULL) {
     uc1601 (" UGII_CAM_LIBRARY_FEEDS_SPEEDS_DATA_DIR -  ", 1 );
     return(-1);
   }

/*          */
  tool_database[0]='\0';
  strcpy(tool_database,ptr1); strcat(tool_database,"tool_database.dat");

  fl=0;
  rf=fopen(tool_database,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  rf=fopen(tool_database,"a");
  if (rf==NULL) { SaveFlag=-1; } else { fclose(rf); }

  if (fl!=0) {
   return(-2);
  }

  UF_print_syslog(tool_database,FALSE);

  tool_materials[0]='\0';
  strcpy(tool_materials,ptr2); strcat(tool_materials,"tool_materials.dat");
  rf=fopen(tool_materials,"r");
  if (rf==NULL) { fl=1; } else { fclose(rf); }

  UF_print_syslog(tool_materials,FALSE);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Loads()
{
   int i ;
   i=Tools_Load();
   if (i<0) return (i);

   i=ToolMaterial_Load();
   if (i<0) ErrLoads( 2 );

 return(0);
}

/*
**********************************************************
**********************************************************
*/
// 1: //CUTS   2: //TOOLMAT   3: //PARTMAT
int ErrLoads( int type)
{
  TOOL_MATERIALS tm; //TOOLMAT

  switch (type) {
   case 1: //CUTS
   break;
   case 2: //TOOLMAT
      tm.Init(); myTM.Add( tm );
   break;
   case 3: //PARTMAT
   break;
   default:break;
  }

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Tools_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr], temp[mStr];
 FILE *rf;
 TOOLS tm;

 rf=fopen(tool_database,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   tool_database\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
   //i=0;
   //while (i<myTM.GetSize()) { delete myTM.GetAt(i++) ; }
   myTOOLS.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.OPERTYPE,stmp,k); //strcat(tm.OPERTYPE,"\0");
	   tm.OPERTYPE[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.PARTMAT,stmp,k); //strcat(tm.PARTMAT,"\0");
	   tm.PARTMAT[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.TOOLMAT,stmp,k); //strcat(tm.TOOLMAT,"\0");
	   tm.TOOLMAT[k]='\0';

	   //DPT_CUT_IN
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //DPT_CUT_MM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.DPT_CUT_MM=atof(temp);

	   //INDEX1
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.INDEX1=atof(temp);

	   //INDEX2
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.INDEX2=atof(temp);

	   //SURF_SPEED_FPM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //SURF_SPEED_MPM
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';
	   tm.SURF_SPEED_MPM=atof(temp);

	   //FEED_IPT
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(temp,stmp,k); //strcat(temp,"\0");
	   temp[k]='\0';

	   //FEED_MMPT
	   temp[0]='\0';
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(temp,stmp,k);
	   } else {
	     strcpy(temp,stmp);
	   }
           //strcat(temp,"\0");
           temp[k]='\0';
           tm.FEED_MMPT=atof(temp);

           myFEEDS.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int ToolMaterial_Load()
{
 int i , k , index;
 char s[mStr], *ptr1, stmp[mStr];
 FILE *rf;
 TOOL_MATERIALS tm;

 rf=fopen(tool_materials,"r");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   tool_materials\n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 ptr1 = (char *) malloc(mStr+10);

 try
 {
    myTM.RemoveAll();
 }
  catch( char * str )
  {
        cerr << "Exception raised: " << str << '\n';
  }

 index=0;

 i=1; s[0]='\0'; fgets(s,mStr,rf);
 while(!feof(rf))
 {
   ptr1=strchr(s,'#');
   if (ptr1==NULL) {
	 stmp[0]='\0'; sprintf(stmp,"%.5s",s);
	 ptr1 = strstr(stmp, "DATA");
	 if (ptr1!=NULL) {

           tm.Init();

           strcpy(tm.FORMAT,"DATA\0");
	   ptr1=strchr(s,'|');
	   if (ptr1==NULL) continue;
	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.LIBREF,stmp,k); //strcat(tm.LIBREF,"\0");
	   tm.LIBREF[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1==NULL) continue;
	   k=ptr1-stmp;
	   strncpy(tm.MATNAM,stmp,k); //strcat(tm.MATNAM,"\0");
	   tm.MATNAM[k]='\0';

	   stmp[0]='\0'; strcpy(stmp,ptr1+1);
	   ptr1=strchr(stmp,'|');
	   if (ptr1!=NULL) continue;
	   ptr1=strchr(stmp,'\n');
	   if (ptr1!=NULL)
	   {
	     k=ptr1-stmp;
	     strncpy(tm.MATDESC,stmp,k);
	   } else {
	     strcpy(tm.MATDESC,stmp);
	   }
           //strcat(tm.MATDESC,"\0");
           tm.MATDESC[k]='\0';

           myTM.Add( tm );
	   index++;
	 }
   }

   //printf("%s ",s);

   i++; s[0]='\0'; fgets(s,mStr,rf);
 }

 fclose(rf);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Saves()
{
 long int index , l;
 int t , st;
 char str[mStr] ;
 FILE *rf;
 TOOLS fd;

 if (SaveFlag==-1) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-1);
  }

 l=myTOOLS.GetSize();
 if (l<=0) {
   uc1601("   \n    - ...",TRUE);
   return(-1);
  }

 index=MessageBox(GetFocus(),(LPCTSTR)" ,      ?\n       -   \n   ...","  ...",MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
 if (index==IDNO) return (0) ;

 rf=fopen(tool_database,"w");
 if (rf==NULL) {
   MessageBox(GetFocus(),(LPCTSTR)"   \n   . "," -  ",MB_ICONSTOP|MB_OK);
   return(-2);
 }

 l=myTOOLS.GetSize();

fputs("#\n",rf);

fputs("#=======================================================================\n",rf);
fputs("#\n",rf);
fputs("#   ASCII Database File   : tool_database.dat\n",rf);
fputs("#   Creation date         : ..................\n",rf);
fputs("#   Unit                  : Metric\n",rf);
fputs("#   Created from Database : ..................\n",rf);
fputs("#\n",rf);
fputs("#=======================================================================\n",rf);

fputs("#\n",rf);
fputs("# UG/Library  Tool Data File\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("# Format of the  ASCII Data File\n",rf);
fputs("# ------------------------------\n",rf);
fputs("#\n",rf);
fputs("# The ASCII Data File is a list of records. The general structure of\n",rf);
fputs("# a record is:\n",rf);
fputs("#\n",rf);
fputs("#    Record_type Paramaters \n",rf);
fputs("#\n",rf);
fputs("# There are three record types which are referred to by the following\n",rf);
fputs("# keywords:\n",rf);
fputs("#\n",rf);
fputs("#    # FORMAT DATA\n",rf);
fputs("#\n",rf);
fputs("# # indicates a comment, the record is ignored.\n",rf);
fputs("#\n",rf);
fputs("# A FORMAT record describes the kind of parameters of the subsequent \n",rf);
fputs("# DATA records.\n",rf);
fputs("#\n",rf);
fputs("# FORMAT LIBRF      T  ST   DESCR        MATREF     MATDES     HLD     HLDDES         DIA     FLEN  FN  HEI \n",rf);
fputs("# \n",rf);
fputs("# This implies that a corresponding DATA record might look like the\n",rf);
fputs("# following example:\n",rf);
fputs("#\n",rf);
fputs("# DATA |ugt0201_001 |02| 01| End Mill |TMC0_00006|HSS-Co5-TiN | 320 | Steep Taper 20 | 10.5 | 35.3| 4| 55 \n",rf);
fputs("#\n",rf);
fputs("# Abbreviations\n",rf);
fputs("#   LIBRF   - Library Reference\n",rf);
fputs("#   T       - Tool Type\n",rf);
fputs("#   ST      - Tool SubType\n",rf);
fputs("#   UGT     - UG Tool Type\n",rf);
fputs("#   UGST    - UG Tool SubType\n",rf);
fputs("#   DESCR   - Description\n",rf);
fputs("#   MATREF  - Reference to cutter material table\n",rf);
fputs("#   MATDES  - Cutter material description\n",rf);
fputs("#   HLD     - Holding system (Type of Machine Adapter)\n",rf);
fputs("#   HLDDES  - Holding system description\n",rf);
fputs("#   DIA     - Diameter\n",rf);
fputs("#   FLEN    - Flute Length\n",rf);
fputs("#   FN      - Number of Flutes\n",rf);
fputs("#   HEI     - Height\n",rf);
fputs("#\n",rf);
fputs("# If a record doesn't start with one of #, FORMAT or DATA it is treated\n",rf);
fputs("# as continuation of the previous FORMAT or DATA record. For example a\n",rf);
fputs("# valid FORMAT description might also be:\n",rf);
fputs("#\n",rf);
fputs("# FORMAT LIBREF      T  ST   UGT   UGST   DESCR MATREF MATDES    HLD   DIA   COR TAPA FLEN\n",rf);
fputs("# # The following 2 Attribute Identifiers also belong to the FORMAT record \n",rf);
fputs("#        FN  HEI \n",rf);
fputs("# \n",rf);
fputs("#\n",rf);
fputs("# A FORMAT record consists of the keyword FORMAT followed by \n",rf);
fputs("# a list of attribute identifiers.\n",rf);
fputs("#\n",rf);
fputs("# A DATA record usually describes one tool. The record consists of the keyword \n",rf);
fputs("# DATA followed by a list of attribute values separated by '|'. \n",rf);
fputs("# The first non blank character after DATA must be a | symbol. \n",rf);
fputs("#\n",rf);
fputs("# The number and kind of attribute fields of a DATA record \n",rf);
fputs("# is described in the preceding FORMAT record.\n",rf);
fputs("# This implies that the  number of attribute values in a DATA record \n",rf);
fputs("# must be the same as the number of attribute identifiers in the \n",rf);
fputs("# corresponding FORMAT record.\n",rf);
fputs("#\n",rf);
fputs("# The attribute identifiers can be used as DB_ID values within the \n",rf);
fputs("# DB_ALIAS structures of the Definition File (dbc_tool_ascii.def).\n",rf);
fputs("# The user can define his own attribute identifiers which he needs to\n",rf);
fputs("# describe his tools. \n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("# The following attributes are common to all tool types and subtypes:\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Unique identifier (so called library refernce) for a tool\n",rf);
fputs("#  T       - Classification type of a tool.\n",rf);
fputs("#  ST      - Classification subtype of a tool.\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Short tool description\n",rf);
fputs("#  MATREF  - Reference to cutter material table (tool_materials.dat)\n",rf);
fputs("#  MATDES  - Cutter material description\n",rf);
fputs("#  HLD     - Holding system for mounting tool to machine\n",rf);
fputs("#            HLD is used to check whether the tool can be mounted\n",rf);
fputs("#            to a pocket of the tool carrier of the machine. \n",rf);
fputs("#            The HLD value here has to match one of the\n",rf);
fputs("#            Holding Systems specified in the pocket definition in\n",rf);
fputs("#            the postprocessor .def file of the machine.\n",rf);
fputs("#  HLDDES  - Holding system description\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("# The attributes LIBRF, T, ST are required!!\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("# The standard UG/Library example uses the attributes T and ST \n",rf);
fputs("# (Type and SubType) in order to classify the tools managed in \n",rf);
fputs("# the ASCII data file.\n",rf);
fputs("# This library classification can be mapped to the UG internal classification \n",rf);
fputs("# using the UGT (UG Type) and UGST (UG SubType) attributes.\n",rf);
fputs("#\n",rf);
fputs("# The following table gives an overview of the available library and UG internal classification\n",rf);
fputs("# and the mapping between them.\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("#    T   Lib Type	   ST  Lib Subtype                         UGT  UG Type        UGST  UG Subtype\n",rf);
fputs("#    --------------|---------------------------------------|--------------------|--------------------\n",rf);
fputs("#    02  Milling     01  End Mills Non-indexable	     01   Mill	         01    5 Parameter\n",rf);
fputs("#    02  Milling     02  End Mills Indexable                 01   Mill           01    5 Parameter\n",rf);
fputs("#    02  Milling     03  Ball Mills Non-indexable            01   Mill           01    5 Parameter\n",rf);
fputs("#    02  Milling     12  Face Mills Indexable                01   Mill           01    5 Parameter\n",rf);
fputs("#    02  Milling     21  T-Slot Mills Non-indexable	     08   T Slot Cut     00    None\n",rf);
fputs("#    02  Milling     93  Barrel Mills                        07   Barrel Cutter  00    None\n",rf);
fputs("#    02  Milling     90  UG -  5 Parameter Cutter	     01   Mill           01    5 Parameter\n",rf);
fputs("#    02  Milling     91  UG -  7 Parameter Cutter	     01   Mill           02    7 Parameter\n",rf);
fputs("#    02  Milling     92  UG - 10 Parameter Cutter	     01   Mill           03    10 Parameter\n",rf);
fputs("#    02  Milling     31  Thread Mills                        02   Drill          10    Thread Mill\n",rf);
fputs("# 	 	 	 \n",rf);
fputs("#    03  Drilling    01 Twist Drills                         02   Drill          00    Standard\n",rf);
fputs("#    03  Drilling    03 Core Drills                          02   Drill          00    Standard\n",rf);
fputs("#    03  Drilling    11 Spot Facing Tools with Pilot	     02   Drill          03    Spot Face\n",rf);
fputs("#    03  Drilling    12 Spot Facing Tools without Pilot      02   Drill          03    Spot Face\n",rf);
fputs("#    03  Drilling    21 Spot Drills	                     02   Drill          04    Spot Drill\n",rf);
fputs("#    03  Drilling    22 Center Drills                        02   Drill          01    Center Bell\n",rf);
fputs("#    03  Drilling    51 Counter Boring Tools Non-indexable   02   Drill          07    Counter Bore\n",rf);
fputs("#    03  Drilling    52 Counter Borring Tools  Indexable     02   Drill          07    Counter Bore\n",rf);
fputs("#    03  Drilling    61 Counter Sinking Tools  Non-indexable 02   Drill          02    Counter Sink\n",rf);
fputs("#    03  Drilling    62 Counter Sinking Tools  Indexable     02   Drill          02    Counter Sink\n",rf);
fputs("#    03  Drilling    71 Taps                                 02   Drill          08    Tap\n",rf);
fputs("#    03  Drilling    31 Adjustable Boring Tools	             02   Drill          05    Bore\n",rf);
fputs("#    03  Drilling    32  Fixed Diameter Boring               02   Drill          05    Bore\n",rf);
fputs("#    03  Drilling    41 Machine Chucking Reamers             02   Drill          06    Drill Ream\n",rf);
fputs("#    03  Drilling    42 Taper Machine Reamers                02   Drill          06    Drill Ream\n",rf);
fputs("#    03  Drilling    04 Step Drills                          02   Drill          12    Step Drill\n",rf);
fputs("#    03  Drilling    90 UG Drills                            02   Drill          00    Standard\n",rf);
fputs("#	 	 	 \n",rf);
fputs("#    01  Turning     01 OD Turning                           03 Turning Tool     01    Standard\n",rf);
fputs("#    01  Turning     02 ID Turning                           03 Turning Tool     01    Standard\n",rf);
fputs("#    01  Turning     11 OD Grooving                          04 Grooving Tool    01    Standard\n",rf);
fputs("#    01  Turning     12 IDGrooving                           04 Grooving Tool    01    Standard\n",rf);
fputs("#    01  Turning     14 Parting                              04 Grooving Tool    01    Standard\n",rf);
fputs("#    01  Turning     21 OD Profiling Tools                   04 Grooving Tool    03    Full Nose Radius\n",rf);
fputs("#    01  Turning     22 ID Profiling Tools                   04 Grooving Tool    03    Full Nose Radius\n",rf);
fputs("#    01  Turning     31 OD Threading Tools                   05 Threading Tool   01    Standard\n",rf);
fputs("#    01  Turning     32 ID Threading Tools                   05 Threading Tool   01    Standard\n",rf);
fputs("#    01  Turning     13 Face Grooving                        04 Grooving Tool    01    Standard\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("# If you want to introduce new types/subtypes in this ASCII data file\n",rf);
fputs("# you also have to specify UGT and UGST in order to map\n",rf);
fputs("# your new tool types to UG internal classification. All available UG types and subtypes\n",rf);
fputs("# are listed in the table above.\n",rf);
fputs("#\n",rf);
fputs("# Usually a DATA record has a unique LIBRF and describes exactly\n",rf);
fputs("# one tool. There is one exception from this rule: If we have a\n",rf);
fputs("# multiple cutter tool we describe every cutter in a separate DATA\n",rf);
fputs("# record. All the data records belonging to a multitool must have the same\n",rf);
fputs("# LIBRF and must have a CTNO (cutter number) attribute. It is \n",rf);
fputs("# not allowed to specify a DATA record with a different LIBRF within \n",rf);
fputs("# such a multitool definition.\n",rf);
fputs("#\n",rf);
fputs("# Currently multiple cutter tools are not supported by the DBC Event Handler.\n",rf);


fputs("#\n",rf);
fputs("# Milling Tools\n",rf);
fputs("# ==============\n",rf);
fputs("#\n",rf);


fputs("#\n",rf);
fputs("#  End Mill (non indexable) \n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset   	\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR             MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI   ZOFF   DROT FLEN TAPA COR1 HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-----------------+----------+-----------+-----+------+---------+---+-----------------+----+--+-----+------+----+----+----+----+-----+----+----+----\n",rf);
 t=2; st=1;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill1(str);
     fputs(str,rf);
   }
 }

fputs("#\n",rf);
fputs("# End Mill (indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA FN HEI   ZOFF     DROT FLEN TAPA COR1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+------+-----+------+---------+---+-----------------+---+--+-----+--------+----+----+----+----+----+----+----+----\n",rf);
 t=2; st=2;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill1(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Ball Mill (non indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                 MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA FN HEI   ZOFF  DROT FLEN TAPA  COR1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+---------------------+----------+-----------+-----+------+---------+---+-----------------+---+--+-----+-----+----+----+-----+----+----+----+----+----\n",rf);
 t=2; st=3;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill1(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Face Mill (indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  D2      - Tool Diameter 2\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR            MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI   ZOFF    DROT FLEN D2   COR1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+----------------+----------+------+-----+------+---------+---+-----------------+----+--+-----+-------+----+----+----+----+----+----+----+----\n",rf);
 t=2; st=12;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill2(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# T Slot Mill (non indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  SDIA    - Tool Shank Diameter\n",rf);
fputs("#  LCOR    - Tool Lower Corner Radius\n",rf);
fputs("#  UCOR    - Tool Upper Corner Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR          MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI  ZOFF  DROT FLEN SDIA LCOR UCOR HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+--------------+----------+-----------+-----+------+---------+---+-----------------+----+--+----+-----+----+----+----+----+----+----+----+----+----\n",rf);
 t=2; st=21;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill3(str);
     fputs(str,rf);
   }
 }

fputs("#\n",rf);
fputs("#  Thread Mill (non indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  THRDS   - Tool Thread Shape (01=ISO Metric 60, 02=Whitworth 55, 03=Trapez 30, 04=Round 30)\n",rf);
fputs("#  THRDES  - Thread Description\n",rf);
fputs("#  PIT     - Tool Pitch\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  SDIA    - Tool Shank Diameter\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI ZOFF  DROT THRDS THRDES                PIT  FLEN TAPA COR1 SDIA  HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+-----------+-----+------+---------+---+-----------------+----+--+---+-----+----+-----+---------------------+----+----+----+----+-----+----+----+----+----\n",rf);
 t=2; st=31;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill_threads(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG 5 Parameter Mill\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG DIA FN HEI ZOFF DROT FLEN TIPA TAPA COR1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+---+--+---+----+----+----+----+----+----+----+----+----+----\n",rf);
 t=2; st=90;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill5(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# 7 Parameter Mill\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  CX1     - Tool Corner1 Center X\n",rf);
fputs("#  CY1     - Tool Corner1 Center Y\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA FN HEI   ZOFF  DROT FLEN TAPA TIPA COR1 CX1 CY1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+------+-----+------+---------+---+-----------------+---+--+-----+-----+----+----+----+----+----+---+---+----+----+----+----\n",rf);
 t=2; st=91;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill7(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# 10 Parameter Mill\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TAPA    - Tool Taper Angle\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  CX1     - Tool Corner1 Center X\n",rf);
fputs("#  CY1     - Tool Corner1 Center Y\n",rf);
fputs("#  COR2    - Tool Corner2 Radius\n",rf);
fputs("#  CX2     - Tool Corner2 Center X\n",rf);
fputs("#  CY2     - Tool Corner2 Center Y\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI ZOFF  DROT FLEN TAPA TIPA COR1 CX1 CY1 COR2 CX2 CY2 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+--------------------+----------+------+-----+------+---------+---+-----------------+----+--+---+-----+----+----+----+----+----+---+---+----+---+---+----+----+----+----\n",rf);
 t=2; st=92;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill10(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Barrel Cutter\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  SDIA    - Tool Shank Diameter\n",rf);
fputs("#  LCOR    - Tool Lower Corner Radius\n",rf);
fputs("#  UCOR    - Tool Upper Corner Radius\n",rf);
fputs("#  BRAD    - Tool Barrel Radius\n",rf);
fputs("#  YCEN    - Tool Barrel Center Y\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG DIA FN HEI ZOFF DROT FLEN SDIA LCOR UCOR HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+---+--+---+----+----+----+----+----+----+----+----+----+----\n",rf);
 t=2; st=93;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_mill3(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Drilling Tools\n",rf);
fputs("# ==============\n",rf);
fputs("#\n",rf);


fputs("#\n",rf);
fputs("# Twist Drill (non indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                 MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA   FN HEI     ZOFF     DROT FLEN PNTA HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+---------------------+----------+------+-----+------+---------+---+-----------------+-----+--+-------+--------+----+----+----+-----+----+----+----\n",rf);
 t=3; st=1;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Twist_Drill_non_indexable(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Core Drill\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA   FN HEI    ZOFF     DROT FLEN HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+-----------+-----+------+---------+---+-----------------+-----+--+------+--------+----+----+-----+----+----+----\n",rf);
 t=3; st=3;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Core_Drill(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Spot Drill\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                 MATREF     MATDES      TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA FN HEI   ZOFF     DROT FLEN PNTA HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+---------------------+----------+-----------+-----+------+---------+---+-----------------+---+--+-----+--------+----+----+----+----+----+----+----\n",rf);
 t=3; st=21;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Spot_Drill(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Center Drill\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  DIA2    - Tool Diameter 2\n",rf);
fputs("#  INCA    - Tool Included Angle\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  FN HEI    ZOFF   DROT FLEN PNTA DIA2 INCA HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+------+-----+------+---------+---+-----------------+----+--+------+------+----+----+----+----+----+-----+----+----+----\n",rf);
 t=3; st=22;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Center_Drill(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Counter Sinking\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter 1\n",rf);
fputs("#  DIA2    - Tool Diameter 2\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR               MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  DIA2 PNTA FN HEI ZOFF   DROT HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------+----------+------+-----+------+---------+---+-----------------+----+----+----+--+---+------+----+-----+----+----+----\n",rf);
 t=3; st=61;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Counter_Sinking(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Tap\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  THRDS   - Tool Thread Shape (01=ISO Metric 60, 02=Whitworth 55, 03=Trapez 30, 04=Round 30)\n",rf);
fputs("#  THRDES  - Thread Description\n",rf);
fputs("#  PIT     - Tool Pitch\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TIPLEN  - Tool Tip Length\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR          MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA THRDS THRDES                PIT  FLEN   TIPLEN HEI    ZOFF     DROT HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+--------------+----------+------+-----+------+---------+---+-----------------+---+-----+---------------------+----+------+------+------+--------+----+----+----+----+----\n",rf);
 t=3; st=71;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Tap(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Chucking Reamer\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TIPLEN  - Tool Tip Length\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                                 MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA   FN HEI    ZOFF     DROT FLEN TIPLEN HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------------------------+----------+------+-----+------+---------+---+-----------------+-----+--+------+--------+----+----+------+-----+----+----+----\n",rf);
 t=3; st=41;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Chucking_Reamer(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Taper Reamer\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  TIPLEN  - Tool Tip Length\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR          MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA    FN HEI   ZOFF   DROT FLEN TIPLEN HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+--------------+----------+------+-----+------+---------+---+-----------------+------+--+-----+------+----+----+------+----+----+----+----\n",rf);
 t=3; st=42;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Taper_Reamer(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Boring (Fixed Diameter)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  COR     - Tool Corner Radius\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  SDIA    - Tool Shank Diameter\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                        MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES           DIA  FN HEI  ZOFF  DROT COR FLEN SDIA HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+----------------------------+----------+------+-----+------+---------+---+----------------+----+--+----+-----+----+---+----+----+----+----+----+----\n",rf);
 t=3; st=32;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Boring_Fixed_Diameter(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Step Drills\n",rf);
fputs("#\n",rf);
fputs("#  Example 2 Steps\n",rf);
fputs("#             _____________________\n",rf);
fputs("#   |         |                |\n",rf);
fputs("#   |---SD2---|                |\n",rf);
fputs("#    \       /________         |\n",rf);
fputs("#     |     |     |            |\n",rf);
fputs("#     |-SD1-|    SL1           |\n",rf);
fputs("#     |     | ____|___         |\n",rf);
fputs("#      \   /_________        FLEN\n",rf);
fputs("#      |   |     |             |\n",rf);
fputs("#      |DIA|  TIPLEN           |\n",rf);
fputs("#      |   |_____|____         |\n",rf);
fputs("#       \ /                    |\n",rf);
fputs("#        V_____________________|___\n",rf);
fputs("#\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Tip Diameter\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  TIPLEN  - Tool Tip Length\n",rf);
fputs("#  COR     - Tool Corner Radius\n",rf);
fputs("#  SD1     - Step 1 Diameter\n",rf);
fputs("#  SA1     - Step 1 Angle\n",rf);
fputs("#  SL1     - Step 1 Length\n",rf);
fputs("#  SR1     - Step 1 Radius\n",rf);
fputs("#  SD2     - Step 2 Diameter\n",rf);
fputs("#  SA2     - Step 2 Angle\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                       MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  PNTA TIPLEN COR SD1 SA1  SL1 SD2   SA2  FLEN FN HEI     ZOFF    DROT HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+---------------------------+----------+------+-----+------+---------+---+-----------------+----+----+------+---+---+----+---+-----+----+----+--+-------+-------+----+-----+----+----+----\n",rf);
 t=3; st=4;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Step_Drills(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Counter Boring (non indexable)\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  PD      - Tool Pilot Diameter\n",rf);
fputs("#  PL      - Tool Pilot Length\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                           MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES            DIA  PD   PL   FLEN FN HEI  ZOFF   DROT HDIA  HLEN HTAP HOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+-------------------------------+----------+------+-----+------+---------+---+-----------------+----+----+----+----+--+----+------+----+-----+----+----+----\n",rf);
 t=3; st=51;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Counter_Boring_non_indexable(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Drill\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  DIA     - Tool Diameter\n",rf);
fputs("#  FN      - Tool Flutes Number\n",rf);
fputs("#  HEI     - Tool Length (Height)\n",rf);
fputs("#  ZOFF    - Tool Z Offset\n",rf);
fputs("#  DROT    - Tool Direction (3=clockwise, 4=counterclockwise)\n",rf);
fputs("#  FLEN    - Tool Flute Length (Cutting Depth)\n",rf);
fputs("#  PNTA    - Tool Point Angle\n",rf);
fputs("#  COR1    - Tool Corner1 Radius\n",rf);
fputs("#  HDIA    - Tool Holder Diameter\n",rf);
fputs("#  HLEN    - Tool Holder Length\n",rf);
fputs("#  HTAP    - Tool Holder Taper\n",rf);
fputs("#  HOFF    - Tool Holder Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG DIA FN HEI ZOFF DROT FLEN PNTA COR1 HDIA HLEN HTAP HOFF\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+---+--+---+----+----+----+----+----+----+----+----+----\n",rf);
 t=3; st=90;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Drill(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Turning Tools\n",rf);
fputs("# ==============\n",rf);
fputs("#\n",rf);


fputs("#\n",rf);
fputs("# OD Turning\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  IT      - Tool Insert Type\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSA    - Tool Nose Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  CLEN    - Tool Cut Edge Length\n",rf);
fputs("#  IC      - Inscribed Circle\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  THCT    - Tool Thickness Type\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  RELT    - Tool Relief Type\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                    MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         IT OA    NOSA NOSR CLEN   IC    THCK THCT RELA RELT MXDP XOFF    YOFF    TP INSP\n",rf);
fputs("# ----+-----------+--+--+---+----+------------------------+----------+------+-----+------+---------+---+--------------+--+-----+----+----+------+-----+----+----+----+----+----+-------+-------+--+----\n",rf);
 t=1; st=1;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_OD_Turning(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# ID Turning\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  IT      - Tool Insert Type\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSA    - Tool Nose Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  CLEN    - Tool Cut Edge Length\n",rf);
fputs("#  IC      - Inscribed Circle\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  THCT    - Tool Thickness Type\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  RELT    - Tool Relief Type\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                      MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         IT OA    NOSA NOSR CLEN   IC     THCK THCT RELA RELT MXDP XOFF     YOFF    TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----------+--+--+---+----+--------------------------+----------+------+-----+------+---------+---+--------------+--+-----+----+----+------+------+----+----+----+----+----+--------+-------+--+----+----+------\n",rf);
 t=1; st=2;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_ID_Turning(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# OD Grooving\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  COR     - Tool Corner Radius\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Left Side Angle\n",rf);
fputs("#  RSA     - Tool Right Side Angle\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  LXOFF   - Tool Left X_Offset\n",rf);
fputs("#  LYOFF   - Tool Left Y_Offset\n",rf);
fputs("#  RXOFF   - Tool Right X_Offset\n",rf);
fputs("#  RYOFF   - Tool Right Y_Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR            MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         OA  IW IL COR THCK MXDP LSA RSA TP INSP LXOFF LYOFF RXOFF   RYOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+----------------+----------+------+-----+------+---------+---+--------------+---+--+--+---+----+----+---+---+--+----+-----+-----+-------+-----\n",rf);
 t=1; st=11;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_OD_Grooving(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# ID Grooving\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  COR     - Tool Corner Radius\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Left Side Angle\n",rf);
fputs("#  RSA     - Tool Right Side Angle\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("#  LXOFF   - Tool Left X_Offset\n",rf);
fputs("#  LYOFF   - Tool Left Y_Offset\n",rf);
fputs("#  RXOFF   - Tool Right X_Offset\n",rf);
fputs("#  RYOFF   - Tool Right Y_Offset\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                    MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         OA   IW   IL COR THCK MXDP LSA RSA TP INSP MNBD MXTR    LXOFF    LYOFF RXOFF    RYOFF\n",rf);
fputs("# ----+-----------+--+--+---+----+------------------------+----------+------+-----+------+---------+---+--------------+----+----+--+---+----+----+---+---+--+----+----+-------+--------+-----+--------+-----\n",rf);
 t=1; st=12;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_ID_Grooving(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# Parting\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  COR     - Tool Corner Radius\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Left Side Angle\n",rf);
fputs("#  RSA     - Tool Right Side Angle\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  PSET    - Tool Preset Cutter  ( R = Right, L = Left )\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR              MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         OA  IW IL COR THCK MXDP LSA RSA TP INSP TIPA XOFF    YOFF    PSET\n",rf);
fputs("# ----+-----------+--+--+---+----+------------------+----------+------+-----+------+---------+---+--------------+---+--+--+---+----+----+---+---+--+----+----+-------+-------+----\n",rf);
 t=1; st=14;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_Parting(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# OD Threading\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  THRDS   - Tool Thread Shape (01=ISO Metric 60, 02=Whitworth 55, 03=Trapez 30, 04=Round 30)\n",rf);
fputs("#  THRDES  - Thread Description\n",rf);
fputs("#  PIT     - Tool Pitch\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  CLEN    - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                   MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         THRDS THRDES                      PIT OA  NOSR CLEN THCK MXDP XOFF    YOFF    TP INSP\n",rf);
fputs("# ----+-----------+--+--+---+----+-----------------------+----------+------+-----+------+---------+---+--------------+-----+---------------------------+---+---+----+----+----+----+-------+-------+--+----\n",rf);
 t=1; st=31;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_OD_Threading(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# ID Threading\n",rf);
fputs("# ----------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  THRDS   - Tool Thread Shape (01=ISO Metric 60, 02=Whitworth 55, 03=Trapez 30, 04=Round 30)\n",rf);
fputs("#  THRDES  - Thread Description\n",rf);
fputs("#  PIT     - Tool Pitch\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  CLEN    - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                   MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         THRDS THRDES                PIT OA   NOSR CLEN THCK MXDP XOFF     YOFF    TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----------+--+--+---+----+-----------------------+----------+------+-----+------+---------+---+--------------+-----+---------------------+---+----+----+----+----+----+--------+-------+--+----+----+------\n",rf);
 t=1; st=32;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_ID_Threading(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# OD Profiling\n",rf);
fputs("# ------------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  RAD     - Tool Radius\n",rf);
fputs("#  LSA     - Tool Side Angle\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                    MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         OA  RAD LSA IL THCK RELA MXDP XOFF    YOFF  TP INSP\n",rf);
fputs("# ----+-----------+--+--+---+----+------------------------+----------+------+-----+------+---------+---+--------------+---+---+---+--+----+----+----+-------+-----+--+----\n",rf);
 t=1; st=21;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_OD_Profiling(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# ID Profiling\n",rf);
fputs("# ------------\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  UGT     - UG Tool Type\n",rf);
fputs("#  UGST    - UG Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  HLD     - Tool Holding System\n",rf);
fputs("#  HLDDES  - Tool Holding System Description\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  RAD     - Tool Radius\n",rf);
fputs("#  LSA     - Tool Side Angle\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF       T  ST UGT UGST DESCR                  MATREF     MATDES TLNUM ADJREG CUTCOMREG HLD HLDDES         OA   RAD LSA IL  THCK RELA MXDP XOFF     YOFF  TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----------+--+--+---+----+----------------------+----------+------+-----+------+---------+---+--------------+----+---+---+---+----+----+----+--------+-----+--+----+----+-----\n",rf);
 t=1; st=22;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_ID_Profiling(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Standard Turning Tool\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  IT      - Tool Insert Type\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSA    - Tool Nose Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  CLEN    - Tool Cut Edge Length\n",rf);
fputs("#  IC      - Inscribed Circle\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  THCT    - Tool Thickness Type\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  RELT    - Tool Relief Type\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG IT OA NOSA NOSR CLEN IC THCK THCT RELA RELT MXDP XOFF YOFF TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+--+----+----+----+--+----+----+----+----+----+----+----+--+----+----+----\n",rf);
 t=1; st=3;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Standard_Turning_Tool(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Turning Tool Button\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  IT      - Tool Insert Type\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  BTDIA   - Tool Button Diameter\n",rf);
fputs("#  BTHA    - Button Holder Angle\n",rf);
fputs("#  BTHW    - Button Holder Width\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  THCT    - Tool Thickness Type\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  RELT    - Tool Relief Type\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG IT OA THCK THCT RELA RELT MXDP XOFF YOFF TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+--+----+----+----+----+----+----+----+--+----+----+----\n",rf);
 t=1; st=3;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Turning_Tool_Button(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Standard Grooving Tool\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  COR     - Tool Left Corner Radius\n",rf);
fputs("#  COR2    - Tool Right Corner Radius\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Side Angle\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  TP      - Tool Left Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  LXOFF   - Tool Left X_Offset\n",rf);
fputs("#  LYOFF   - Tool Left Y_Offset\n",rf);
fputs("#  RXOFF   - Tool Right X_Offset\n",rf);
fputs("#  RYOFF   - Tool Right Y_Offset\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA IW IL COR COR2 THCK MXDP LSA TIPA TP INSP LXOFF LYOFF RXOFF RYOFF MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+--+--+---+----+----+----+---+----+--+----+-----+-----+-----+-----+----+----\n",rf);
 t=1; st=4;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Standard_Grooving_Tool(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Grooving Tool (Full Nose Radius)\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  LSA     - Tool Side Angle\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  RELA    - Tool Relief Angle\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Left Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA LSA IL THCK RELA MXDP XOFF YOFF TP INSP MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+---+--+----+----+----+----+----+--+----+----+----\n",rf);
 t=1; st=4;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Grooving_Tool_Full_Nose_Radius(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Grooving Tool (Ring Type Joint)\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSW    - Tool Nose Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Left Angle\n",rf);
fputs("#  RSA     - Tool Right Angle\n",rf);
fputs("#  TP      - Tool Left Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  LXOFF   - Tool Left X_Offset\n",rf);
fputs("#  LYOFF   - Tool Left Y_Offset\n",rf);
fputs("#  RXOFF   - Tool Right X_Offset\n",rf);
fputs("#  RYOFF   - Tool Right Y_Offset\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA IL THCK MXDP LSA RSA TP INSP LXOFF LYOFF RXOFF RYOFF MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+--+----+----+---+---+--+----+-----+-----+-----+-----+----+----\n",rf);
 t=1; st=4;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Grooving_Tool_Ring_Type_Joint(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Grooving Tool (user defined)\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  LSA     - Tool Left Angle\n",rf);
fputs("#  RSA     - Tool Right Angle\n",rf);
fputs("#  TIPA    - Tool Tip Angle\n",rf);
fputs("#  COR     - Tool Left Corner Radius\n",rf);
fputs("#  COR2    - Tool Right Corner Radius\n",rf);
fputs("#  TP      - Tool Left Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("#  LXOFF   - Tool Left X_Offset\n",rf);
fputs("#  LYOFF   - Tool Left Y_Offset\n",rf);
fputs("#  RXOFF   - Tool Right X_Offset\n",rf);
fputs("#  RYOFF   - Tool Right Y_Offset\n",rf);
fputs("#  MNBD    - Tool Min Boring Diameter\n",rf);
fputs("#  MXTR    - Tool Max Toolreach\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA IW IL THCK MXDP LSA RSA TIPA COR COR2 TP INSP LXOFF LYOFF RXOFF RYOFF MNBD MXTR\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+--+--+----+----+---+---+----+---+----+--+----+-----+-----+-----+-----+----+----\n",rf);
 t=1; st=4;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Grooving_Tool_user_defined(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Threading Tool (Standard)\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSR    - Tool Nose Radius\n",rf);
fputs("#  LA      - Tool Left Angle\n",rf);
fputs("#  RA      - Tool Right Angle\n",rf);
fputs("#  TOFF    - Tool Tip Offset\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA NOSR THCK MXDP XOFF YOFF TP INSP\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+----+----+----+----+----+--+----\n",rf);
 t=3; st=5;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Threading_Tool_Standard(str);
     fputs(str,rf);
   }
 }


fputs("#\n",rf);
fputs("# UG Threading Tool (Trapez)\n",rf);
fputs("# This type is for legacy tools which were converted from old\n",rf);
fputs("# partfile tool libraries\n",rf);
fputs("#\n",rf);
fputs("#  LIBRF   - Tool Library Reference\n",rf);
fputs("#  T       - Tool Type\n",rf);
fputs("#  ST      - Tool SubType\n",rf);
fputs("#  DESCR   - Description\n",rf);
fputs("#  MATREF  - Tool Material Code\n",rf);
fputs("#  MATDES  - Tool Material Description\n",rf);
fputs("#  TLNUM   - Tool Number\n",rf);
fputs("#  ADJREG  - Tool Length Adjust Register\n",rf);
fputs("#  CUTCOMREG - Tool CUTCOM Register\n",rf);
fputs("#  OA      - Tool Orientation Angle\n",rf);
fputs("#  NOSW    - Tool Nose Width\n",rf);
fputs("#  LA      - Tool Left Angle\n",rf);
fputs("#  RA      - Tool Right Angle\n",rf);
fputs("#  TOFF    - Tool Tip Offset\n",rf);
fputs("#  IW      - Tool Insert Width\n",rf);
fputs("#  IL      - Tool Insert Length\n",rf);
fputs("#  THCK    - Tool Thickness\n",rf);
fputs("#  MXDP    - Tool Max Depth\n",rf);
fputs("#  XOFF    - Tool X_Offset\n",rf);
fputs("#  YOFF    - Tool Y_Offset\n",rf);
fputs("#  TP      - Tool Tracking Point\n",rf);
fputs("#  INSP    - Tool Insert Position  (1=topside, 2=downside)\n",rf);
fputs("\n",rf);
fputs("FORMAT LIBRF T ST DESCR MATREF MATDES TLNUM ADJREG CUTCOMREG OA THCK MXDP XOFF YOFF TP INSP\n",rf);
fputs("# ----+-----+-+--+-----+------+------+-----+------+---------+--+----+----+----+----+--+----\n",rf);
 t=1; st=5;
 for (index=0;index<l;index++)
 {
   fd = myTOOLS.GetAt(index);
   if (fd.T==t && fd.ST==st) {
     str[0]='\0'; fd.FormatFile_UG_Threading_Tool_Trapez(str);
     fputs(str,rf);
   }
 }


 fputs("\n\n\n",rf);

 fclose(rf);

 str[0]='\0'; sprintf(str," ...\n   %ld - .", l );
 MessageBox(GetFocus(),(LPCTSTR)str," ...",MB_ICONINFORMATION|MB_OK);

 return(0);
}


/*
**********************************************************
**********************************************************
*/
int Deletes(int index)
{
 if (index<0 || index>=myTOOLS.GetSize()) return (-1);

 try
 {
   myTOOLS.RemoveAt(index);
 }
  catch( char * str )
  {
    cerr << "Exception raised: " << str << '\n';
  }

 return (0);
}


/*
**********************************************************
**********************************************************
*/
char * NewInd(int t, int st)
{
   int index ;
   long int i;
   char ls[25], *ptr ;
   TOOLS  fd;

   /* Seed the random-number generator with current time so that
    * the numbers will be different every time we run. */
  srand( (unsigned)time( NULL ) );
  ls[0]='\0'; sprintf(ls,"ugt%02d%02d_%03d",t,st,rand() );

  for (i=0,index=0;index<myTOOLS.GetSize();index++)
  {
    fd = myTOOLS.GetAt(index);
    if (0==strcmp( ls, fd.LIBREF)) {
    	if (i>100000) break;
    	ls[0]='\0'; sprintf(ls,"ugt%02d%02d_%03d",t,st,rand() );
    	i++;
    	continue;
     }
  }

  ptr = (char *) malloc(strlen(ls)+5);
  strcpy(ptr,ls);strcat(ptr,"\0");
 return(ptr);
}


/*
**********************************************************
**********************************************************
*/
int FindIndex( int type,long int index)
{
  long int i, step, len;
  char ls[26];
  TOOLS  fd;
  TOOL_MATERIALS tm; //TOOLMAT

  fd = myTOOLS.GetAt(index);

  ls[0]='\0';
  len=-1;
  switch (type) {
   case 1: //CUTS
   break;
   case 2: //TOOLMAT
   len=myTM.GetSize();   sprintf(ls,"%.25s",strupr(fd.TOOLMAT) );
   break;
   case 3: //PARTMAT
   break;
   default:break;
  }

  for (step=-1,i=0;i<len;i++)
  {

    if (type==1) { //CUTS
    }
    if (type==2) { //TOOLMAT
    	tm=myTM.GetAt(i);
        if (NULL!=strstr( ls, strupr(tm.LIBREF))) { step=i;break; }
    }
    if (type==3) { //PARTMAT
    }

  }

  if (step<0) step=0;
  return(step);
}

/*
**********************************************************
**********************************************************
*/
char * RecordIndex( int type,long int i)
{
  long int len;
  char ls[26], *ptr ;
  TOOL_MATERIALS tm; //TOOLMAT

  ls[0]='\0';
  switch (type) {
   case 1: //CUTS
   break;
   case 2: //TOOLMAT
    len=myTM.GetSize();
    if (i<0 || i>=len) { sprintf(ls,"NULL"); break; }
    tm=myTM.GetAt(i);
    sprintf(ls,"%.20s",strupr(tm.LIBREF) );
   break;
   case 3: //PARTMAT
   break;
   default:break;
  }

  ptr = (char *) malloc(strlen(ls)+1);
  strcpy(ptr,ls);strcat(ptr,"\0");
  return(ptr);
}

